@@ -4,9 +4,6 @@ namespace App;
 
 use Respect\Validation\Validator;
 
-use Movim\Picture;
-use Movim\User;
-
 use Illuminate\Database\Eloquent\Model;
 use Illuminate\Database\Capsule\Manager as DB;
 
@@ -64,7 +61,7 @@ class Post extends Model
     {
         return $this->hasMany('App\Attachment')
                     ->where('category', 'link')
-                    ->whereNotIn('href', function($query) {
+                    ->whereNotIn('href', function ($query) {
                         $query->select('href')
                               ->from('attachments')
                               ->where('post_id', $this->id)
@@ -109,9 +106,13 @@ class Post extends Model
         try {
             parent::save($options);
 
-            $this->attachments()->delete();
-            $this->attachments()->saveMany($this->attachments);
-            $this->tags()->sync($this->tags);
+            if (!$this->isComment()) {
+                $this->healAttachments();
+
+                $this->attachments()->delete();
+                $this->attachments()->saveMany($this->attachments);
+                $this->tags()->sync($this->tags);
+            }
         } catch (\Exception $e) {
             /*
              * When an article is received by two accounts simultaenously
@@ -141,10 +142,10 @@ class Post extends Model
         $configuration = Configuration::get();
 
         if ($configuration->restrictsuggestions) {
-            $query->whereIn('id', function($query) {
+            $query->whereIn('id', function ($query) {
                 $host = \App\User::me()->session->host;
                 $query->select('id')
-                        ->from('posts')
+                      ->from('posts')
                       ->where('server', 'like', '%.' . $host)
                       ->orWhere('server', 'like', '@' . $host);
             });
@@ -162,12 +163,13 @@ class Post extends Model
 
     public function scopeRecents($query)
     {
-        $query->join(DB::raw('(
+        $query->join(
+            DB::raw('(
             select max(published) as published, server, node
             from posts
             group by server, node) as recents
-            '), function($join)
-            {
+            '),
+            function ($join) {
                 $join->on('posts.node', '=', 'recents.node');
                 $join->on('posts.published', '=', 'recents.published');
             }
@@ -176,7 +178,7 @@ class Post extends Model
 
     protected function withContactsScope($query)
     {
-        return $query->orWhereIn('posts.server', function($query) {
+        return $query->orWhereIn('posts.server', function ($query) {
             $query->from('rosters')
                   ->select('jid')
                   ->where('session_id', SESSION_ID)
@@ -191,7 +193,7 @@ class Post extends Model
 
     protected function withMineScope($query)
     {
-        return $query->orWhereIn('id', function($query) {
+        return $query->orWhereIn('id', function ($query) {
             $query->select('id')
                   ->from('posts')
                   ->where('node', 'urn:xmpp:microblog:0')
@@ -206,16 +208,16 @@ class Post extends Model
 
     protected function withSubscriptionsScope($query)
     {
-        return $query->orWhereIn('id', function($query) {
+        return $query->orWhereIn('id', function ($query) {
             $query->select('id')
                   ->from('posts')
-                  ->whereIn('server', function($query) {
-                    $query->select('server')
+                  ->whereIn('server', function ($query) {
+                      $query->select('server')
                           ->from('subscriptions')
                           ->where('jid', \App\User::me()->id);
                   })
-                  ->whereIn('node', function($query) {
-                    $query->select('node')
+                  ->whereIn('node', function ($query) {
+                      $query->select('node')
                           ->from('subscriptions')
                           ->where('jid', \App\User::me()->id);
                   });
@@ -232,6 +234,7 @@ class Post extends Model
         return \App\Post::where('server', $this->server)
                        ->where('node', $this->node)
                        ->where('published', '<', $this->published)
+                       ->where('open', true)
                        ->orderBy('published', 'desc')
                        ->first();
     }
@@ -242,6 +245,7 @@ class Post extends Model
                        ->where('node', $this->node)
                        ->where('published', '>', $this->published)
                        ->orderBy('published')
+                       ->where('open', true)
                        ->first();
     }
 
@@ -251,8 +255,7 @@ class Post extends Model
             return $this->contact->truename;
         }
 
-        $truename = explodeJid($this->aid);
-        return (isset($truename['username'])) ? $truename['username'] : '';
+        return explodeJid($this->aid)['username'] ?? '';
     }
 
     private function extractContent($contents)
@@ -260,7 +263,7 @@ class Post extends Model
         $content = '';
 
         foreach ($contents as $c) {
-            switch($c->attributes()->type) {
+            switch ($c->attributes()->type) {
                 case 'html':
                 case 'xhtml':
                     $dom = new \DOMDocument('1.0', 'utf-8');
@@ -277,7 +280,7 @@ class Post extends Model
                         $this->contentraw = trim($c);
                     }
                     break;
-                default :
+                default:
                     $content = (string)$c;
                     break;
             }
@@ -290,7 +293,7 @@ class Post extends Model
     {
         $title = '';
         foreach ($titles as $t) {
-            switch($t->attributes()->type) {
+            switch ($t->attributes()->type) {
                 case 'html':
                 case 'xhtml':
                     $title = strip_tags((string)$t->children()->asXML());
@@ -300,7 +303,7 @@ class Post extends Model
                         $title = trim($t);
                     }
                     break;
-                default :
+                default:
                     $title = (string)$t;
                     break;
             }
@@ -314,20 +317,27 @@ class Post extends Model
         $this->nodeid = (string)$entry->attributes()->id;
 
         // Get some informations on the author
-        if ($entry->entry->author->name)
+        if ($entry->entry->author->name) {
             $this->aname = (string)$entry->entry->author->name;
-        if ($entry->entry->author->uri)
+        }
+        if ($entry->entry->author->uri) {
             $this->aid = substr((string)$entry->entry->author->uri, 5);
-        if ($entry->entry->author->email)
+        }
+        if ($entry->entry->author->email) {
             $this->aemail = (string)$entry->entry->author->email;
+        }
 
         // Non standard support
-        if ($entry->entry->source && $entry->entry->source->author->name)
+        if ($entry->entry->source && $entry->entry->source->author->name) {
             $this->aname = (string)$entry->entry->source->author->name;
-        if ($entry->entry->source && $entry->entry->source->author->uri)
+        }
+        if ($entry->entry->source && $entry->entry->source->author->uri) {
             $this->aid = substr((string)$entry->entry->source->author->uri, 5);
+        }
 
-        if (empty($this->aname)) $this->aname = null;
+        if (empty($this->aname)) {
+            $this->aname = null;
+        }
 
         $this->title = $this->extractTitle($entry->entry->title);
 
@@ -348,17 +358,19 @@ class Post extends Model
 
         $this->updated = ($entry->entry->updated)
             ? toSQLDate($entry->entry->updated)
-            : gmdate(SQL_DATE);
+            : gmdate(MOVIM_SQL_DATE);
 
         if ($entry->entry->published) {
             $this->published = toSQLDate($entry->entry->published);
         } elseif ($entry->entry->updated) {
             $this->published = toSQLDate($entry->entry->updated);
         } else {
-            $this->published = gmdate(SQL_DATE);
+            $this->published = gmdate(MOVIM_SQL_DATE);
         }
 
-        if ($delay) $this->delay = $delay;
+        if ($delay) {
+            $this->delay = $delay;
+        }
 
         // Tags parsing
         if ($entry->entry->category) {
@@ -370,7 +382,9 @@ class Post extends Model
 
                 $this->tags[] = $tag->id;
 
-                if ($tag->name == 'nsfw') $this->nsfw = true;
+                if ($tag->name == 'nsfw') {
+                    $this->nsfw = true;
+                }
             } else {
                 foreach ($entry->entry->category as $cat) {
                     $tag = \App\Tag::firstOrCreateSafe([
@@ -379,7 +393,9 @@ class Post extends Model
 
                     $this->tags[] = $tag->id;
 
-                    if ($tag->tag == 'nsfw') $this->nsfw = true;
+                    if ($tag->tag == 'nsfw') {
+                        $this->nsfw = true;
+                    }
                 }
             }
         }
@@ -395,7 +411,9 @@ class Post extends Model
             $this->tags[] = $tag->id;
         }
 
-        if (current(explode('.', $this->server)) == 'nsfw') $this->nsfw = true;
+        if (current(explode('.', $this->server)) == 'nsfw') {
+            $this->nsfw = true;
+        }
 
         if (!isset($this->commentserver)) {
             $this->commentserver = $this->server;
@@ -469,7 +487,7 @@ class Post extends Model
             $enc = (array)$attachment->attributes();
             $enc = $enc['@attributes'];
 
-            $att = new \App\Attachment;
+            $att = new Attachment;
 
             if (empty($enc['href']) || empty($enc['rel'])) {
                 continue;
@@ -479,8 +497,12 @@ class Post extends Model
             $att->href = $enc['href'];
             $att->category = 'other';
 
-            if (isset($enc['title'])) $att->title = $enc['title'];
-            if (isset($enc['description'])) $att->description = $enc['description'];
+            if (isset($enc['title'])) {
+                $att->title = $enc['title'];
+            }
+            if (isset($enc['description'])) {
+                $att->description = $enc['description'];
+            }
 
             switch ($enc['rel']) {
                 case 'enclosure':
@@ -505,7 +527,7 @@ class Post extends Model
                     $att->logo = (isset($enc['logo'])) ? $enc['logo'] : null;
 
                     if (preg_match('%(?:youtube(?:-nocookie)?\.com/(?:[^/]+/.+/|(?:v|e(?:mbed)?)/|.*[?&]v=)|youtu\.be/)([^"&?/ ]{11})%i', $enc['href'], $match)) {
-                        $atty = new \App\Attachment;
+                        $atty = new Attachment;
                         $atty->rel = $enc['rel'];
                         $atty->href = 'https://www.youtube.com/embed/' . $match[1];
                         $atty->category = 'youtube';
@@ -527,7 +549,7 @@ class Post extends Model
         }
 
         if ($picture == false && $extra) {
-            $attachment = new \App\Attachment;
+            $attachment = new Attachment;
             $attachment->rel = 'enclosure';
             $attachment->href = $extra;
             $attachment->type = 'content';
@@ -536,6 +558,23 @@ class Post extends Model
         }
     }
 
+    private function healAttachments()
+    {
+        $enclosures = [];
+
+        foreach (array_filter($this->attachments, function ($a) { return $a->rel == 'enclosure'; }) as $attachment)
+        {
+            array_push($enclosures, $attachment->href);
+        }
+
+        foreach (array_filter($this->attachments, function ($a) { return $a->rel != 'enclosure'; }) as $key => $attachment)
+        {
+            if (in_array($attachment->href, $enclosures)) {
+                unset($this->attachments[$key]);
+            }
+        }
+    }
+
     public function getUUID()
     {
         if (substr($this->nodeid, 10) == 'urn:uuid:') {
@@ -638,7 +677,9 @@ class Post extends Model
 
     public function getReply()
     {
-        if (!$this->replynodeid) return;
+        if (!$this->replynodeid) {
+            return;
+        }
 
         return \App\Post::where('server', $this->replyserver)
                         ->where('node', $this->replynode)
