@@ -5,11 +5,6 @@ use App\Configuration;
 
 class AdminloginController extends Base
 {
-    public function load()
-    {
-        $this->session_only = false;
-    }
-
     public function dispatch()
     {
         $this->page->setTitle(__('page.administration'));
