@@ -7,11 +7,6 @@ class InfosController extends Base
     public function load()
     {
         header('Content-type: application/json');
-        $this->session_only = false;
         $this->raw = true;
     }
-
-    public function dispatch()
-    {
-    }
 }
