@@ -1,7 +1,6 @@
 <?php
 
 use Movim\Controller\Base;
-use Movim\User;
 
 class NewsController extends Base
 {
@@ -17,9 +16,5 @@ class NewsController extends Base
         if (!\App\User::me()->hasPubsub()) {
             $this->redirect('contact');
         }
-
-        if (!\App\User::me()->isLogged()) {
-            $this->redirect('login');
-        }
     }
 }
