@@ -3,7 +3,6 @@
 use Monolog\Logger;
 use Monolog\Handler\SyslogHandler;
 use Monolog\Handler\StreamHandler;
-use GuzzleHttp\Client;
 use Movim\Picture;
 
 class Utils
@@ -22,7 +21,9 @@ class Utils
             $log->pushHandler($handler);
 
             $errlines = explode("\n", $message);
-            foreach ($errlines as $txt) { $log->addInfo($txt); }
+            foreach ($errlines as $txt) {
+                $log->info($txt);
+            }
         }
     }
 
@@ -34,11 +35,26 @@ class Utils
         $log = new Logger('movim');
         $log->pushHandler(new StreamHandler(LOG_PATH . '/debug.log'));
         if (is_array($logs)) {
-            $log->addDebug('', $logs);
+            $log->debug('', $logs);
         } else {
-            $log->addDebug($logs);
+            $log->debug($logs);
         }
     }
+
+    /**
+     * Log a string, only used for debug purposes
+     */
+    public static function error($logs)
+    {
+        $log = new Logger('movim');
+        $log->pushHandler(new SyslogHandler('movim'));
+
+        if (defined('LOG_LEVEL') && LOG_LEVEL > 1) {
+            $log->pushHandler(new StreamHandler(LOG_PATH . '/errors.log'));
+        }
+
+        $log->error($logs);
+    }
 }
 
 
@@ -49,6 +65,7 @@ function getClientTypes()
 {
     return [
         'bot'           => __('client.bot'),
+        'console'       => __('client.console'),
         'pc'            => __('client.desktop'),
         'phone'         => __('client.phone'),
         'handheld'      => __('client.phone'),
@@ -387,7 +404,8 @@ function getCountries()
     ];
 }
 
-function getPresences() {
+function getPresences()
+{
     return [
             1 => __('presence.online'),
             2 => __('presence.away'),
@@ -396,7 +414,6 @@ function getPresences() {
             5 => __('presence.offline'),
             6 => __('presence.error')
         ];
-
 }
 
 function getPresencesTxt()
@@ -501,6 +518,41 @@ function getMood()
     ];
 }
 
+/**
+ * Map the XMPP form vars to Material icons
+ */
+function varToIcons(string $var)
+{
+    $icons = [
+        // Pubsub
+        'pubsub#deliver_payloads' => 'add_box',
+        'pubsub#notify_config' => 'notifications',
+        'pubsub#notify_delete' => 'delete',
+        'pubsub#notify_retract' => 'delete_sweep',
+        'pubsub#persist_items' => 'save',
+        'pubsub#deliver_notifications' => 'notifications_active',
+
+        // Muc
+        'muc#roomconfig_persistentroom' => 'save',
+        'muc#roomconfig_publicroom' => 'wifi_tethering',
+        'muc#roomconfig_passwordprotectedroom' => 'lock',
+        'muc#roomconfig_membersonly' => 'playlist_add_check',
+        'muc#roomconfig_moderatedroom' => 'stars',
+        'muc#roomconfig_changesubject' => 'title',
+        'muc#roomconfig_allowinvites' => 'mail',
+        'allow_visitor_status' => 'description',
+        'allow_private_messages' => 'message',
+        'allow_query_users' => 'portrait',
+        'mam' => 'archive',
+    ];
+
+    if (array_key_exists($var, $icons)) {
+        return $icons[$var];
+    }
+
+    return 'check_box';
+}
+
 /**
  * Generate a standard UUID
  */
@@ -524,7 +576,7 @@ function generateKey($size)
     $hash_chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
     $hash = "";
 
-    for($i = 0; $i < $size; $i++) {
+    for ($i = 0; $i < $size; $i++) {
         $r = mt_rand(0, strlen($hash_chars) - 1);
         $hash.= $hash_chars[$r];
     }
@@ -552,8 +604,8 @@ function requestURL(string $url, int $timeout = 10, $post = false, bool $json =
     }
 
     if (is_array($post)) {
-        curl_setopt ($ch, CURLOPT_POST, 1);
-        curl_setopt ($ch, CURLOPT_POSTFIELDS, $post);
+        curl_setopt($ch, CURLOPT_POST, 1);
+        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
     }
 
     $content = curl_exec($ch);
@@ -592,8 +644,8 @@ function requestAPI(string $action, int $timeout = 2, $post = false)
     curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
 
     if (is_array($post)) {
-        curl_setopt ($ch, CURLOPT_POST, 1);
-        curl_setopt ($ch, CURLOPT_POSTFIELDS, $post);
+        curl_setopt($ch, CURLOPT_POST, 1);
+        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
     }
 
     $content = curl_exec($ch);
@@ -605,7 +657,7 @@ function requestAPI(string $action, int $timeout = 2, $post = false)
  */
 function getSmileyPath($id)
 {
-    return BASE_URI.'/themes/material/img/emojis/svg/'.$id.'.svg';
+    return BASE_URI . 'theme/img/emojis/svg/'.$id.'.svg';
 }
 
 /*
