@@ -26,7 +26,7 @@ class Caps extends Base
 
     public function display()
     {
-        $clients = App\Capability::where('category', 'client')->orderBy('name')->get();
+        $clients = \App\Info::whereCategory('client')->orderBy('name')->get();
         $oldname = '';
 
         foreach ($clients as $c) {
@@ -34,7 +34,9 @@ class Caps extends Base
             $parts = explode('#', reset($parts));
             $clientname = reset($parts);
 
-            if ($oldname == $clientname) continue;
+            if ($oldname == $clientname) {
+                continue;
+            }
 
             if (!isset($this->_table[$c->name])) {
                 $this->_table[$c->name] = [];
