@@ -1,43 +1,44 @@
 <?php
 
-use Moxl\Xec\Action\Message\Composing;
-use Moxl\Xec\Action\Message\Paused;
 use Moxl\Xec\Action\Message\Publish;
+use Moxl\Xec\Action\Message\Reactions;
+use Moxl\Xec\Action\Message\Retract;
 
 use Moxl\Xec\Action\Muc\GetConfig;
 use Moxl\Xec\Action\Muc\SetConfig;
 
 use App\Configuration;
+use App\Message;
+use App\Reaction;
 
 use Moxl\Xec\Action\BOB\Request;
+use Moxl\Xec\Action\Disco\Request as DiscoRequest;
 
 use Respect\Validation\Validator;
 
 use Illuminate\Database\Capsule\Manager as DB;
 
 use Movim\Picture;
-use Movim\Session;
 use Movim\ChatStates;
-
-include_once WIDGETS_PATH.'ContactActions/ContactActions.php';
+use Movim\ChatOwnState;
 
 class Chat extends \Movim\Widget\Base
 {
     private $_pagination = 50;
     private $_wrapper = [];
-    private $_mucPresences = [];
+    private $_messageTypes = ['chat', 'headline', 'invitation', 'jingle_start', 'jingle_end'];
 
     public function load()
     {
         $this->addjs('chat.js');
         $this->addcss('chat.css');
-        $this->registerEvent('invitation', 'onMessage');
         $this->registerEvent('carbons', 'onMessage');
         $this->registerEvent('message', 'onMessage');
+        $this->registerEvent('retracted', 'onRetracted');
         $this->registerEvent('receiptack', 'onMessageReceipt');
         $this->registerEvent('displayed', 'onMessage', 'chat');
-        //$this->registerEvent('mamresult', 'onMessageHistory', 'chat');
-        $this->registerEvent('mam_get_handle', 'onMAMRetrieved', 'chat');
+        $this->registerEvent('mam_get_handle', 'onMAMRetrieved');
+        $this->registerEvent('mam_get_handle_muc', 'onMAMMucRetrieved', 'chat');
         $this->registerEvent('composing', 'onComposing', 'chat');
         $this->registerEvent('paused', 'onPaused', 'chat');
         $this->registerEvent('subject', 'onConferenceSubject', 'chat');
@@ -46,34 +47,10 @@ class Chat extends \Movim\Widget\Base
         $this->registerEvent('muc_setconfig_handle', 'onRoomConfigSaved', 'chat');
         $this->registerEvent('muc_setconfig_error', 'onRoomConfigError', 'chat');
         $this->registerEvent('presence_muc_handle', 'onMucConnected', 'chat');
+        $this->registerEvent('message_publish_error', 'onPublishError', 'chat');
 
         $this->registerEvent('bob_request_handle', 'onSticker');
-        //$this->registerEvent('presence', 'onPresence');
-    }
-
-    /*
-     * Disabled for the moment, it SPAM a bit too much the user
-    public function onPresence($packet)
-    {
-        $contacts = $packet->content;
-        if ($contacts != null) {
-            $contact = $contacts[0];
-
-            if ($contact->value < 5) {
-                $avatar = $contact->getPhoto();
-                if ($avatar == false) $avatar = null;
-
-                $presences = getPresences();
-                $presence = $presences[$contact->value];
-
-                Notification::append('presence', $contact->truename, $presence, $avatar, 4);
-            }
-        }
-    }*/
-
-    public function onMessageHistory($packet)
-    {
-        $this->onMessage($packet, true);
+        $this->registerEvent('notification_counter_clear', 'onNotificationCounterClear');
     }
 
     public function onMessageReceipt($packet)
@@ -81,33 +58,50 @@ class Chat extends \Movim\Widget\Base
         $this->onMessage($packet, false, true);
     }
 
+    public function onRetracted($packet)
+    {
+        $this->onMessage($packet, false, true);
+    }
+
+    public function onNotificationCounterClear($params)
+    {
+        list($page, $first, $room) = array_pad($params, 3, null);
+
+        if ($page === 'chat') {
+            $this->prepareMessages($first, ($room === 'room'), true);
+        }
+    }
+
+    public function onPublishError($packet)
+    {
+        Notification::toast(
+            $packet->content ??
+            $this->__('chat.publish_error')
+        );
+    }
+
     public function onMessage($packet, $history = false, $receipt = false)
     {
         $message = $packet->content;
         $from = null;
         $chatStates = ChatStates::getInstance();
 
-        if ($message->isEmpty()) return;
+        if ($message->isEmpty()) {
+            return;
+        }
 
         if ($message->user_id == $message->jidto
         && !$history
+        && $message->seen == false
         && $message->jidfrom != $message->jidto) {
             $from = $message->jidfrom;
-            $roster = $this->user->session->contacts()->where('jid', $from)->first();
             $contact = App\Contact::firstOrNew(['id' => $from]);
 
-            // If the message was displayed by another of my clients
-            if ($message->user_id == $message->jidto
-            && $message->displayed) {
-                $n = new Notification;
-                $n->ajaxClear('chat|'.$message->jidfrom);
-            }
-            // If the message is for me
-            elseif ($contact != null
-            //&& $message->isTrusted()
-            && !$message->isOTR()
+            if ($contact != null
+            && !$message->encrypted
             && $message->type != 'groupchat'
             && !$message->oldid) {
+                $roster = $this->user->session->contacts()->where('jid', $from)->first();
                 $chatStates->clearState($from);
 
                 Notification::append(
@@ -123,7 +117,6 @@ class Chat extends \Movim\Widget\Base
             elseif ($message->type == 'groupchat'
                    && $message->quoted
                    && !$receipt) {
-
                 $conference = $this->user->session
                                    ->conferences()->where('conference', $from)
                                    ->first();
@@ -135,21 +128,20 @@ class Chat extends \Movim\Widget\Base
                         : $from,
                     $message->resource.': '.$message->body,
                     false,
-                    4);
+                    4
+                );
             } elseif ($message->type == 'groupchat') {
                 $chatStates->clearState($from, $message->resource);
             }
 
             $this->onPaused($chatStates->getState($from));
-        } else {
-            // If the message is from me we reset the notif counter
-            $n = new Notification;
-            $n->ajaxClear('chat|'.$message->jidto);
         }
 
-        if (!$message->isOTR()) {
+        if (!$message->encrypted) {
             $this->rpc('Chat.appendMessagesWrapper', $this->prepareMessage($message, $from));
         }
+
+        $this->event('chat_counter', $this->user->unreads());
     }
 
     public function onSticker($packet)
@@ -183,9 +175,14 @@ class Chat extends \Movim\Widget\Base
         $this->ajaxGetRoom($packet->content->jidfrom);
     }
 
-    public function onMAMRetrieved($packet)
+    public function onMAMRetrieved()
     {
-        $this->ajaxGetRoom($packet->content);
+        Notification::toast($this->__('chat.mam_retrieval'));
+    }
+
+    public function onMAMMucRetrieved($packet)
+    {
+        $this->ajaxGetRoom($packet->content, true, true);
     }
 
     public function onMucConnected($packet)
@@ -195,7 +192,7 @@ class Chat extends \Movim\Widget\Base
 
     public function onRoomConfigError($packet)
     {
-        Notification::append(false, $packet->content);
+        Notification::toast($packet->content);
     }
 
     public function onRoomConfig($packet)
@@ -215,7 +212,11 @@ class Chat extends \Movim\Widget\Base
 
     public function onRoomConfigSaved($packet)
     {
-        Notification::append(false, $this->__('chatroom.config_saved'));
+        $r = new DiscoRequest;
+        $r->setTo($packet->content)
+          ->request();
+
+        Notification::toast($this->__('chatroom.config_saved'));
     }
 
     private function setState(string $jid, string $message)
@@ -232,6 +233,12 @@ class Chat extends \Movim\Widget\Base
         $this->rpc('Chat.setGeneralElements', $date, $separator);
     }
 
+    public function ajaxClearCounter($jid)
+    {
+        $this->prepareMessages($jid, false, true);
+        $this->event('chat_counter', $this->user->unreads());
+    }
+
     /**
      * @brief Get a discussion
      * @param string $jid
@@ -239,10 +246,12 @@ class Chat extends \Movim\Widget\Base
     public function ajaxGet($jid = null, $light = false)
     {
         if ($jid == null) {
+            $this->rpc('MovimTpl.hidePanel');
             $this->rpc('Notification.current', 'chat');
             $this->rpc('MovimUtils.pushState', $this->route('chat'));
-            $this->rpc('MovimTpl.fill', '#chat_widget', $this->prepareEmpty());
-            $this->rpc('MovimTpl.hidePanel');
+            if ($light == false) {
+                $this->rpc('MovimTpl.fill', '#chat_widget', $this->prepareEmpty());
+            }
         } else {
             if ($light == false) {
                 $this->rpc('MovimUtils.pushState', $this->route('chat', $jid));
@@ -262,7 +271,9 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxGetRoom($room, $light = false, $noConnect = false)
     {
-        if (!$this->validateJid($room)) return;
+        if (!$this->validateJid($room)) {
+            return;
+        }
 
         $r = $this->user->session->conferences()->where('conference', $room)->first();
 
@@ -279,24 +290,12 @@ class Chat extends \Movim\Widget\Base
             }
 
             $this->prepareMessages($room, true);
-
-            $notif = new Notification;
-            $notif->ajaxClear('chat|'.$room);
-            $this->rpc('Notification.current', 'chat|'.$room);
+            $this->rpc('Notification.current', 'chat|'.$room.'|room');
         } else {
             $this->rpc('Rooms_ajaxAdd', $room);
         }
     }
 
-    /**
-     * @brief Get a Drawer view of a contact
-     */
-    public function ajaxGetContact($jid)
-    {
-        $c = new ContactActions;
-        $c->ajaxGetDrawer($jid);
-    }
-
     /**
      * @brief Send a message
      *
@@ -304,13 +303,14 @@ class Chat extends \Movim\Widget\Base
      * @param string $message
      * @return void
      */
-    public function ajaxHttpSendMessage($to, $message = false, $muc = false, $resource = false, $replace = false, $file = false)
+    public function ajaxHttpDaemonSendMessage($to, $message = false, $muc = false, $resource = false, $replace = false, $file = false)
     {
         $message = trim($message);
         if (filter_var($message, FILTER_VALIDATE_URL)) {
             $headers = requestHeaders($message);
 
             if ($headers['http_code'] == 200
+            && isset($headers['content_type'])
             && typeIsPicture($headers['content_type'])
             && $headers['download_content_length'] > 100) {
                 $file = new \stdClass;
@@ -321,7 +321,7 @@ class Chat extends \Movim\Widget\Base
             }
         }
 
-        $body = ($file != false)
+        $body = ($file != false && $file->type != 'xmpp')
             ? $file->uri
             : $message;
 
@@ -344,6 +344,7 @@ class Chat extends \Movim\Widget\Base
         } else {
             $m = new \App\Message;
             $m->id          = generateUUID();
+            $m->originid    = generateUUID();
             $m->replaceid   = $m->id;
             $m->user_id     = $this->user->id;
             $m->jidto       = echapJid($to);
@@ -353,6 +354,7 @@ class Chat extends \Movim\Widget\Base
 
         // TODO: make this boolean configurable
         $m->markable = true;
+        $m->seen = true;
 
         $m->type    = 'chat';
         $m->resource = $this->user->session->resource;
@@ -380,6 +382,7 @@ class Chat extends \Movim\Widget\Base
         }
 
         $p->setId($m->id);
+        $p->setOriginid($m->originid);
 
         if ($muc) {
             $p->setMuc();
@@ -390,6 +393,8 @@ class Chat extends \Movim\Widget\Base
             $p->setFile($file);
         }
 
+        (ChatOwnState::getInstance())->halt();
+
         $p->request();
 
         /* Is it really clean ? */
@@ -397,9 +402,15 @@ class Chat extends \Movim\Widget\Base
             $m->oldid = $oldid;
             $m->body = htmlentities(trim($m->body), ENT_XML1, 'UTF-8');
             $m->save();
+            $m = $m->fresh();
 
             $packet = new \Moxl\Xec\Payload\Packet;
             $packet->content = $m;
+
+            // We refresh the Chats list
+            $c = new Chats;
+            $c->onMessage($packet);
+
             $this->onMessage($packet);
         }
     }
@@ -411,15 +422,103 @@ class Chat extends \Movim\Widget\Base
      * @param string $message
      * @return void
      */
-    public function ajaxHttpCorrect($to, $message)
+    public function ajaxHttpDaemonCorrect($to, $message, $mid = false)
     {
-        $replace = $this->user->messages()
-                        ->where('jidto', $to)
-                        ->orderBy('published', 'desc')
-                        ->first();
+        $replace = $mid
+            ? $this->user->messages()
+                         ->where('mid', $mid)
+                         ->first()
+            : $this->user->messages()
+                         ->where('jidto', $to)
+                         ->orderBy('published', 'desc')
+                         ->first();
 
         if ($replace) {
-            $this->ajaxHttpSendMessage($to, $message, false, false, $replace);
+            $this->ajaxHttpDaemonSendMessage($to, $message, false, false, $replace);
+        }
+    }
+
+    /**
+     * @brief Send a reaction
+     */
+    public function ajaxHttpDaemonSendReaction($mid, string $emoji)
+    {
+        $parentMessage = $this->user->messages()
+                        ->where('mid', $mid)
+                        ->first();
+
+        $emojiHandler = \Movim\Emoji::getInstance();
+        $emojiHandler->replace($emoji);
+
+        if ($parentMessage && $emojiHandler->isSingleEmoji()) {
+            // Try to load the MUC presence and resolve the resource
+            $mucPresence = null;
+            if ($parentMessage->type == 'groupchat') {
+                $mucPresence = $this->user->session->presences()
+                                    ->where('jid', $parentMessage->jidfrom)
+                                    ->where('mucjid', $this->user->id)
+                                    ->where('muc', true)
+                                    ->first();
+
+                if (!$mucPresence) return;
+            }
+
+            $jidfrom = ($parentMessage->type == 'groupchat')
+                ? $mucPresence->resource
+                : $this->user->id;
+
+            $emojis = $parentMessage->reactions()
+                ->where('jidfrom', $jidfrom)
+                ->get();
+
+            $r = new Reactions;
+            $newEmojis = [];
+
+            // This reaction was not published yet
+            if ($emojis->where('emoji', $emoji)->count() == 0) {
+                $reaction = new Reaction;
+                $reaction->message_mid = $parentMessage->mid;
+                $reaction->jidfrom = ($parentMessage->type == 'groupchat')
+                    ? $this->user->session->username
+                    : $this->user->id;
+                $reaction->emoji = $emoji;
+
+                if ($parentMessage->type != 'groupchat') {
+                    $reaction->save();
+                }
+
+                $newEmojis = $emojis->push($reaction);
+            } else {
+                if ($parentMessage->type != 'groupchat') {
+                    $parentMessage->reactions()
+                        ->where('jidfrom', $jidfrom)
+                        ->where('emoji', $emoji)
+                        ->delete();
+                }
+
+                $newEmojis = $emojis->filter(function ($value, $key) use ($emoji) {
+                    return $value->emoji != $emoji;
+                });
+            }
+
+            $r->setTo($parentMessage->jidfrom != $parentMessage->user_id
+                ? $parentMessage->jidfrom
+                : $parentMessage->jidto)
+              ->setId(\generateUUID())
+              ->setParentId($parentMessage->replaceid)
+              ->setReactions($newEmojis->pluck('emoji')->toArray());
+
+            if ($parentMessage->type == 'groupchat') {
+                $r->setMuc();
+            }
+
+            $r->request();
+
+            if ($parentMessage->type != 'groupchat') {
+                $packet = new \Moxl\Xec\Payload\Packet;
+                $packet->content = $parentMessage;
+                $this->onMessage($packet);
+            }
         }
     }
 
@@ -441,6 +540,23 @@ class Chat extends \Movim\Widget\Base
             $this->rpc('Chat.setTextarea', htmlspecialchars_decode($m->body));
         }
     }
+    /**
+     * @brief Get the a sent message
+     *
+     * @param string $mid
+     * @return void
+     */
+    public function ajaxEdit($mid)
+    {
+        $m = $this->user->messages()
+                        ->where('mid', $mid)
+                        ->first();
+
+        if (!isset($m->sticker)
+        && !isset($m->file)) {
+            $this->rpc('Chat.setTextarea', htmlspecialchars_decode($m->body), $mid);
+        }
+    }
 
     /**
      * @brief Send a "composing" message
@@ -450,34 +566,11 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxSendComposing($to, $muc = false)
     {
-        if (!$this->validateJid($to)) return;
-
-        $mc = new Composing;
-
-        if ($muc) {
-            $mc->setMuc();
+        if (!$this->validateJid($to)) {
+            return;
         }
 
-        $mc->setTo($to)->request();
-    }
-
-    /**
-     * @brief Send a "paused" message
-     *
-     * @param string $to
-     * @return void
-     */
-    public function ajaxSendPaused($to, $muc = false)
-    {
-        if (!$this->validateJid($to)) return;
-
-        $mp = new Paused;
-
-        if ($muc) {
-            $mp->setMuc();
-        }
-
-        $mp->setTo($to)->request();
+        (ChatOwnState::getInstance())->composing($to, $muc);
     }
 
     /**
@@ -488,19 +581,21 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxGetHistory($jid, $date, $muc = false, $prepend = true)
     {
-        if (!$this->validateJid($jid)) return;
+        if (!$this->validateJid($jid)) {
+            return;
+        }
 
         $messages = $this->user->messages()
                          ->where(function ($query) use ($jid) {
-                                $query->where('jidfrom', $jid)
+                             $query->where('jidfrom', $jid)
                                       ->orWhere('jidto', $jid);
                          })
-                         ->where('published', $prepend ? '<' : '>', date(SQL_DATE, strtotime($date)));
+                         ->where('published', $prepend ? '<' : '>', date(MOVIM_SQL_DATE, strtotime($date)));
 
 
         $messages = $muc
             ? $messages->where('type', 'groupchat')->whereNull('subject')
-            : $messages->whereIn('type', ['chat', 'headline', 'invitation']);
+            : $messages->whereIn('type', $this->_messageTypes);
 
         $messages = $messages->orderBy('published', 'desc')
                              ->take($this->_pagination)
@@ -508,13 +603,13 @@ class Chat extends \Movim\Widget\Base
 
         if ($messages->count() > 0) {
             if ($prepend) {
-                Notification::append(false, $this->__('message.history', $messages->count()));
+                Notification::toast($this->__('message.history', $messages->count()));
             } else {
                 $messages = $messages->reverse();
             }
 
             foreach ($messages as $message) {
-                if (!$message->isOTR()) {
+                if (!$message->encrypted) {
                     $this->prepareMessage($message);
                 }
             }
@@ -531,7 +626,9 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxGetRoomConfig($room)
     {
-        if (!$this->validateJid($room)) return;
+        if (!$this->validateJid($room)) {
+            return;
+        }
 
         $gc = new GetConfig;
         $gc->setTo($room)
@@ -545,7 +642,9 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxSetRoomConfig($data, $room)
     {
-        if (!$this->validateJid($room)) return;
+        if (!$this->validateJid($room)) {
+            return;
+        }
 
         $sc = new SetConfig;
         $sc->setTo($room)
@@ -558,7 +657,9 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxDisplayed($jid, $id)
     {
-        if (!$this->validateJid($jid)) return;
+        if (!$this->validateJid($jid)) {
+            return;
+        }
 
         $message = $this->user->messages()->where('id', $id)->first();
 
@@ -579,7 +680,9 @@ class Chat extends \Movim\Widget\Base
      */
     public function ajaxClearHistory($jid)
     {
-        if (!$this->validateJid($jid)) return;
+        if (!$this->validateJid($jid)) {
+            return;
+        }
 
         $this->user->messages()->where(function ($query) use ($jid) {
             $query->where('jidfrom', $jid)
@@ -599,10 +702,12 @@ class Chat extends \Movim\Widget\Base
         $view->assign('emoji', prepareString('😀'));
         $view->assign('muc', $muc);
         $view->assign('anon', false);
-        $view->assign('info',
+        $view->assign(
+            'info',
             \App\Info::where('server', $this->user->session->host)
                      ->where('node', '')
-                     ->first());
+                     ->first()
+        );
 
         if ($muc) {
             $view->assign('room', $jid);
@@ -625,47 +730,65 @@ class Chat extends \Movim\Widget\Base
         return $view->draw('_chat');
     }
 
-    public function prepareMessages($jid, $muc = false)
+    public function prepareMessages($jid, $muc = false, $seenOnly = false)
     {
-        if (!$this->validateJid($jid)) return;
+        if (!$this->validateJid($jid)) {
+            return;
+        }
 
         $jid = echapJid($jid);
 
-        $messages = $this->user->messages()->where(function ($query) use ($jid) {
+        $messagesQuery = $this->user->messages()->where(function ($query) use ($jid) {
             $query->where('jidfrom', $jid)
                   ->orWhere('jidto', $jid);
         });
 
-        $messages = $muc
-            ? $messages->where('type', 'groupchat')->whereNull('subject')
-            : $messages->whereIn('type', ['chat', 'headline', 'invitation']);
+        $messagesQuery = $muc
+            ? $messagesQuery->where('type', 'groupchat')->whereNull('subject')
+            : $messagesQuery->whereIn('type', $this->_messageTypes);
 
-        $messages = $messages->orderBy('published', 'desc')->take($this->_pagination)->get();
-        $messages = $messages->reverse();
+        /**
+         * The object need to be cloned there for MySQL, looks like the pagination/where is kept somewhere in between…
+         **/
+        $messagesRequest = clone $messagesQuery;
+        $messagesCount = clone $messagesQuery;
 
-        foreach ($messages as $message) {
-            $this->prepareMessage($message);
+        $messages = $messagesRequest->orderBy('published', 'desc')->take($this->_pagination)->get();
+        $unreadsCount = $messagesCount->where('seen', false)->count();
+
+        if ($unreadsCount > 0) {
+            $messagesClear = clone $messagesQuery;
+            $messagesClear->where('seen', false)->update(['seen' => true]);
         }
 
-        $view = $this->tpl();
-        $view->assign('jid', $jid);
+        if (!$seenOnly) {
+            $messages = $messages->reverse();
 
-        $view->assign('contact', \App\Contact::firstOrNew(['id' => $jid]));
-        $view->assign('me', false);
-        $view->assign('muc', $muc);
-        $left = $view->draw('_chat_bubble');
+            foreach ($messages as $message) {
+                $this->prepareMessage($message);
+            }
 
-        $view->assign('contact', \App\Contact::firstOrNew(['id' => $this->user->id]));
-        $view->assign('me', true);
-        $view->assign('muc', $muc);
-        $right = $view->draw('_chat_bubble');
+            $view = $this->tpl();
+            $view->assign('jid', $jid);
 
-        $this->rpc('Chat.setSpecificElements', $left, $right);
-        $this->rpc('Chat.appendMessagesWrapper', $this->_wrapper, false, true);
+            $view->assign('contact', \App\Contact::firstOrNew(['id' => $jid]));
+            $view->assign('me', false);
+            $view->assign('muc', $muc);
+            $left = $view->draw('_chat_bubble');
 
-        $notif = new Notification;
-        $this->rpc('Chat.insertSeparator', $notif->getCounter('chat|'.$jid));
-        $notif->ajaxClear('chat|'.$jid);
+            $view->assign('contact', \App\Contact::firstOrNew(['id' => $this->user->id]));
+            $view->assign('me', true);
+            $view->assign('muc', $muc);
+            $right = $view->draw('_chat_bubble');
+
+            $this->rpc('Chat.setSpecificElements', $left, $right);
+            $this->rpc('Chat.appendMessagesWrapper', $this->_wrapper, false, true);
+        }
+
+        $this->event($muc ? 'chat_open_room' : 'chat_open', $jid);
+        $this->event('chat_counter', $this->user->unreads());
+
+        $this->rpc('Chat.insertSeparator', $unreadsCount);
     }
 
     public function prepareMessage(&$message, $jid = null)
@@ -679,7 +802,11 @@ class Chat extends \Movim\Widget\Base
 
         $emoji = \Movim\Emoji::getInstance();
 
-        if (isset($message->html)) {
+        if ($message->retracted) {
+            $message->body = '<i class="material-icons">delete</i> '.__('message.retracted');
+        } elseif ($message->encrypted) {
+            $message->body = '<i class="material-icons">lock</i> '.__('message.encrypted');
+        } elseif (isset($message->html) && !isset($message->file)) {
             $message->body = $message->html;
         } else {
             $message->addUrls();
@@ -714,17 +841,21 @@ class Chat extends \Movim\Widget\Base
         }
 
         // Jumbo emoji
-        if ($emoji->isSingleEmoji()) {
+        if ($emoji->isSingleEmoji()
+            && !isset($message->html)
+            && in_array($message->type,  ['chat', 'groupchat'])) {
             $message->sticker = [
                 'url' => $emoji->getLastSingleEmojiURL(),
-                'height' => 60
+                'title' => ':'.$emoji->getLastSingleEmojiTitle().':',
+                'height' => 60,
             ];
         }
 
         // Attached file
         if (isset($message->file)) {
             // We proxify pictures links even if they are advertized as small ones
-            if (typeIsPicture($message->file['type'])
+            if (\array_key_exists('type', $message->file)
+            && typeIsPicture($message->file['type'])
             && $message->file['size'] <= SMALL_PICTURE_LIMIT) {
                 $message->sticker = [
                     'thumb' => $this->route('picture', urlencode($message->file['uri'])),
@@ -733,25 +864,48 @@ class Chat extends \Movim\Widget\Base
                 ];
             }
 
-            // Other image websites
             $url = parse_url($message->file['uri']);
+            // Other image websites
+            if (\array_key_exists('host', $url)) {
+                switch ($url['host']) {
+                    case 'i.imgur.com':
+                        $matches = [];
+                        preg_match('/https:\/\/i.imgur.com\/([a-zA-Z0-9]{7})(.*)/', $message->file['uri'], $matches);
 
-            switch ($url['host']) {
-                case 'i.imgur.com':
-                    $matches = [];
-                    preg_match('/https:\/\/i.imgur.com\/([a-zA-Z0-9]{7})(.*)/', $message->file['uri'], $matches);
+                        if (!empty($matches)) {
+                            $message->sticker = [
+                                'url' => $message->file['uri'],
+                                'thumb' => 'https://i.imgur.com/' . $matches[1] . 'g' . $matches[2],
+                                'picture' => true
+                            ];
+                        }
+                        break;
+                }
+            }
 
-                    if (!empty($matches)) {
-                        $message->sticker = [
-                            'url' => $message->file['uri'],
-                            'thumb' => 'https://i.imgur.com/' . $matches[1] . 'g' . $matches[2],
-                            'picture' => true
-                        ];
+            // Build cards for the URIs
+            $uri = explodeXMPPURI($message->file['uri']);
+
+            switch ($uri['type']) {
+                case 'post':
+                    $post = \App\Post::where('server', $uri['params'][0])
+                        ->where('node',  $uri['params'][1])
+                        ->where('nodeid',  $uri['params'][2])
+                        ->first();
+
+                    if ($post) {
+                        $p = new Post;
+                        $message->card = $p->prepareTicket($post);
                     }
                     break;
             }
         }
 
+        // Reactions
+        if ($message->reactions()->count()) {
+            $message->reactionsHtml = $this->prepareReactions($message);
+        }
+
         $message->rtl = isRTL($message->body);
         $message->publishedPrepared = prepareTime(strtotime($message->published));
 
@@ -765,13 +919,18 @@ class Chat extends \Movim\Widget\Base
 
         $date = prepareDate(strtotime($message->published), false, false, true);
 
-        if (empty($date)) $date = $this->__('date.today');
+        if (empty($date)) {
+            $date = $this->__('date.today');
+        }
 
         // We create the date wrapper
         if (!array_key_exists($date, $this->_wrapper)) {
             $this->_wrapper[$date] = [];
         }
 
+        $messageDBSeen = $message->seen;
+        $n = new Notification;
+
         if ($message->type == 'groupchat') {
             $message->color = stringToColor($message->session_id . $message->resource . $message->type);
 
@@ -791,16 +950,30 @@ class Chat extends \Movim\Widget\Base
                 }
 
                 $message->moderator = ($this->mucPresences[$key]->mucrole == 'moderator');
-                $message->mine = ($this->mucPresences[$key]->mucjid == $this->user->id);
+                $message->mine = $message->seen = ($this->mucPresences[$key]->mucjid == $this->user->id);
+
             } else {
                 $this->mucPresences[$key] = true;
             }
 
             $message->icon = firstLetterCapitalize($message->resource);
+
+            if ($message->seen === false) {
+                $message->seen = ('chat|'.$message->jidfrom.'|room' == $n->getCurrent());
+            }
+        } elseif($message->seen === false) {
+            $message->seen = ('chat|'.$message->jidfrom == $n->getCurrent());
+        }
+
+        if ($message->seen === true
+        && $messageDBSeen === false) {
+            $this->user->messages()
+                 ->where('id', $message->id)
+                 ->update(['seen' => true]);
         }
 
         $msgkey = '<' . $message->jidfrom;
-        $msgkey .= ($message->type == 'groupchat') ? $message->resource : '';
+        $msgkey .= ($message->type == 'groupchat') ? cleanupId($message->resource, true) : '';
         $msgkey .= '>' . substr($message->published, 11, 5);
 
         $counter = count($this->_wrapper[$date]);
@@ -813,26 +986,82 @@ class Chat extends \Movim\Widget\Base
             $message->body = $view->draw('_chat_invitation');
         }
 
+        if ($message->type == 'jingle_start') {
+            $view = $this->tpl();
+            $view->assign('message', $message);
+            $message->body = $view->draw('_chat_jingle_start');
+        }
+
+        if ($message->type == 'jingle_end') {
+            $view = $this->tpl();
+            $view->assign('message', $message);
+            $view->assign('diff', false);
+
+            $start = Message::where(
+                [
+                    'type' =>'jingle_start',
+                    'thread'=> $message->thread
+                ]
+            )->first();
+
+            if ($start) {
+                $diff = (new DateTime($start->created_at))
+                  ->diff(new DateTime($message->created_at));
+
+                $view->assign('diff', $diff);
+            }
+
+            $message->body = $view->draw('_chat_jingle_end');
+        }
+
         return $this->_wrapper;
     }
 
+    public function prepareReactions(Message $message)
+    {
+        $view = $this->tpl();
+        $merged = [];
+
+        $reactions = $message
+            ->reactions()
+            ->orderBy('created_at')
+            ->get();
+
+        foreach ($reactions as $reaction) {
+            if (!array_key_exists($reaction->emoji, $merged)) {
+                $merged[$reaction->emoji] = [];
+            }
+
+            $merged[$reaction->emoji][] = $reaction->jidfrom;
+        }
+
+        $view->assign('message', $message);
+        $view->assign('reactions', $merged);
+        $view->assign('me', $this->user->id);
+
+        return $view->draw('_chat_reactions');
+    }
+
     public function prepareEmpty()
     {
         $view = $this->tpl();
 
-        $conferences = \App\Info::where('category', 'conference')
+        $conferences = \App\Info::whereCategory('conference')
                                 ->whereNotIn('server', $this->user->session->conferences()->pluck('conference')->toArray())
+                                ->restrictUserHost()
                                 ->where('mucpublic', true)
                                 ->where('mucpersistent', true);
 
-        $conferences = (Configuration::get()->restrictsuggestions)
-            ? $conferences->where('server', 'like', '%@%.' . $this->user->session->host)
-            : $conferences->where('server', 'like', '%@%');
+        if (Configuration::get()->restrictsuggestions) {
+            $conferences = $conferences->where('server', 'like', '%@%.' . $this->user->session->host);
+        }
 
         $conferences = $conferences->orderBy('occupants', 'desc')->take(8)->get();
 
         $chats = \App\Cache::c('chats');
-        if ($chats == null) $chats = [];
+        if ($chats == null) {
+            $chats = [];
+        }
         $chats[$this->user->id] = true;
 
         $top = $this->user->session->topContacts()
@@ -867,7 +1096,7 @@ class Chat extends \Movim\Widget\Base
      */
     private function validateJid($jid)
     {
-        return (Validator::stringType()->noWhitespace()->length(6, 60)->validate($jid));
+        return (Validator::stringType()->noWhitespace()->length(6, 256)->validate($jid));
     }
 
     public function getSmileyPath($id)
