@@ -9,7 +9,7 @@ var Chat = {
     edit: false,
 
     // Chat state
-    state: 0,
+    composing: false,
     since: null,
     sended: false,
 
@@ -18,6 +18,13 @@ var Chat = {
     lastAutocomplete: null,
     searchAutocomplete: null,
 
+    // Touch
+    startX: 0,
+    startY: 0,
+    translateX: 0,
+    translateY: 0,
+    slideAuthorized: false,
+
     autocomplete: function(event, jid)
     {
         Rooms_ajaxMucUsersAutocomplete(jid);
@@ -106,6 +113,9 @@ var Chat = {
         var muc = Boolean(textarea.dataset.muc);
         var jid = textarea.dataset.jid;
 
+        // In case it was in edit mode
+        textarea.classList.remove('edit');
+
         textarea.focus();
 
         if (!Chat.sended) {
@@ -117,9 +127,14 @@ var Chat = {
 
             if (Chat.edit) {
                 Chat.edit = false;
-                xhr = Chat_ajaxHttpCorrect(jid, text);
+                if (textarea.dataset.mid) {
+                    xhr = Chat_ajaxHttpDaemonCorrect(jid, text, textarea.dataset.mid);
+                    delete textarea.dataset.mid;
+                } else {
+                    xhr = Chat_ajaxHttpDaemonCorrect(jid, text);
+                }
             } else {
-                xhr = Chat_ajaxHttpSendMessage(jid, text, muc);
+                xhr = Chat_ajaxHttpDaemonSendMessage(jid, text, muc);
             }
 
             xhr.onreadystatechange = function() {
@@ -169,14 +184,29 @@ var Chat = {
             Chat_ajaxLast(textarea.dataset.jid);
         }
     },
+    editMessage: function(mid)
+    {
+        var textarea = Chat.getTextarea();
+        if (textarea.value == ''
+        && Boolean(textarea.dataset.muc) == false) {
+            Chat_ajaxEdit(mid);
+        }
+    },
     focus: function()
     {
         Chat.sended = false;
+        Chat.composing = false;
         Chat.clearReplace();
         Chat.toggleAction();
 
         var textarea = Chat.getTextarea();
         textarea.onkeydown = function(event) {
+            if ((event.keyCode == 37 && Chat.tryPreviousEmoji())
+             || (event.keyCode == 39 && Chat.tryNextEmoji())) {
+                event.preventDefault();
+                return;
+            }
+
             if (this.dataset.muc
             && event.keyCode == 9) {
                 event.preventDefault();
@@ -192,23 +222,32 @@ var Chat = {
                 Chat.editPrevious();
             } else if (event.keyCode == 40
             && (this.value == '' || Chat.edit == true)) {
+                localStorage.removeItem(textarea.dataset.jid + '_message');
                 Chat.clearReplace();
             }
         };
 
         textarea.onkeypress = function(event) {
             if (event.keyCode == 13) {
+                // An emoji was selected
+                var emoji = document.querySelector('.chat_box .emojis img.selected');
+                if (emoji) {
+                    Chat.selectEmoji(emoji);
+                    event.preventDefault();
+                    return;
+                }
+
                 if ((window.matchMedia('(max-width: 1024px)').matches && !event.shiftKey)
                 || (window.matchMedia('(min-width: 1025px)').matches && event.shiftKey)) {
                     return;
                 }
 
-                Chat.state = 0;
+                Chat.composing = false;
                 Chat.sendMessage();
 
                 return false;
-            } else if (Chat.state == 0 || Chat.state == 2) {
-                Chat.state = 1;
+            } else if (Chat.composing === false) {
+                Chat.composing = true;
                 Chat_ajaxSendComposing(this.dataset.jid, Boolean(this.dataset.muc));
                 Chat.since = new Date().getTime();
             }
@@ -216,17 +255,14 @@ var Chat = {
 
         textarea.onkeyup = function(event) {
             localStorage.setItem(this.dataset.jid + '_message', this.value);
+
+            // A little timeout to not spam the server with composing states
             setTimeout(function()
             {
-                var textarea = document.querySelector('#chat_textarea');
-
-                if (textarea
-                && Chat.state == 1
-                && Chat.since + 5000 < new Date().getTime()) {
-                    Chat.state = 2;
-                    Chat_ajaxSendPaused(textarea.dataset.jid, Boolean(textarea.dataset.muc));
+                if (Chat.since + 3000 < new Date().getTime()) {
+                    Chat.composing = false;
                 }
-            },5000);
+            }, 3000);
 
             Chat.toggleAction();
         };
@@ -235,6 +271,7 @@ var Chat = {
             var discussion = Chat.getDiscussion();
             var scrolled = (discussion.scrollHeight - discussion.scrollTop === discussion.clientHeight);
             MovimUtils.textareaAutoheight(this);
+            Chat.checkEmojis(this);
 
             if (scrolled) discussion.scrollTop = discussion.scrollHeight;
         }
@@ -249,15 +286,86 @@ var Chat = {
 
         Chat.autocompleteList = null;
     },
-    setTextarea: function(value)
+    checkEmojis: function(textarea)
+    {
+        var emojisList = document.querySelector('.chat_box .emojis');
+        emojisList.innerHTML = '';
+
+        if (textarea.value.lastIndexOf(':') > -1 && textarea.value.length > textarea.value.lastIndexOf(':') + 2) {
+            var first = true;
+
+            Object.keys(emojis).filter(key => key.indexOf(
+                    textarea.value.substr(textarea.value.lastIndexOf(':') + 1)
+                ) > -1)
+            .slice(0, 40)
+            .forEach(found => {
+                var img = document.createElement('img');
+                img.setAttribute('src','theme/img/emojis/svg/' + emojis[found].c + '.svg');
+                img.classList.add('emoji');
+
+                if (first) {
+                    img.classList.add('selected');
+                    first = false;
+                }
+
+                img.title = ':' + found + ':';
+                img.dataset.emoji = emojis[found].e;
+                img.addEventListener('click', e => {
+                    Chat.selectEmoji(e.target);
+                });
+                emojisList.appendChild(img);
+            });
+        }
+    },
+    selectEmoji: function(emoji)
+    {
+        var emojisList = document.querySelector('.chat_box .emojis');
+        var textarea = Chat.getTextarea();
+
+        textarea.value = textarea.value.substr(0, textarea.value.lastIndexOf(':'));
+        emojisList.innerHTML = '';
+        Chat.insertAtCursor(emoji.dataset.emoji + ' ');
+    },
+    tryNextEmoji: function()
+    {
+        var currentEmoji = document.querySelector('.chat_box .emojis img.selected');
+
+        if (currentEmoji && currentEmoji.nextSibling) {
+            currentEmoji.classList.remove('selected');
+            currentEmoji.nextSibling.classList.add('selected');
+            return true;
+        }
+
+        return false;
+    },
+    tryPreviousEmoji: function()
+    {
+        var currentEmoji = document.querySelector('.chat_box .emojis img.selected');
+
+        if (currentEmoji && currentEmoji.previousSibling) {
+            currentEmoji.classList.remove('selected');
+            currentEmoji.previousSibling.classList.add('selected');
+            return true;
+        }
+
+        return false;
+    },
+    setTextarea: function(value, mid)
     {
         Chat.edit = true;
         var textarea = Chat.getTextarea();
         textarea.value = value;
+        textarea.classList.add('edit');
+
+        if (mid) {
+            textarea.dataset.mid = mid;
+        }
+
         MovimUtils.textareaAutoheight(textarea);
         textarea.focus();
     },
-    setGeneralElements(date, separator) {
+    setGeneralElements(date, separator)
+    {
         var div = document.createElement('div');
 
         Chat.currentDate = null;
@@ -267,7 +375,8 @@ var Chat = {
         div.innerHTML = separator;
         Chat.separator = div.firstChild.cloneNode(true);
     },
-    setSpecificElements : function(left, right) {
+    setSpecificElements : function(left, right)
+    {
         var div = document.createElement('div');
 
         Chat.currentDate = null;
@@ -277,8 +386,12 @@ var Chat = {
         div.innerHTML = right;
         Chat.right = div.firstChild.cloneNode(true);
     },
-    setScrollBehaviour : function() {
+    setScrollBehaviour : function()
+    {
         var discussion = Chat.getDiscussion();
+
+        if (discussion == null) return;
+
         discussion.onscroll = function() {
             if (this.scrollTop < 1
             && discussion.querySelectorAll('ul li div.bubble p').length >= Chat.pagination) {
@@ -292,7 +405,34 @@ var Chat = {
             Chat.lastHeight = this.clientHeight;
         };
     },
-    checkDiscussion : function(page) {
+    setReactionButtonBehaviour : function()
+    {
+        let reactions = document.querySelectorAll('#chat_widget span.reaction');
+        let i = 0;
+
+        while (i < reactions.length) {
+            reactions[i].onclick = function() {
+                Stickers_ajaxReaction(this.dataset.mid);
+            }
+
+            i++;
+        }
+    },
+    setActionsButtonBehaviour : function()
+    {
+        let actions = document.querySelectorAll('#chat_widget .contained:not(.muc) span.actions');
+        let i = 0;
+
+        while (i < actions.length) {
+            actions[i].onclick = function() {
+                ChatActions_ajaxShowMessageDialog(this.dataset.mid);
+            }
+
+            i++;
+        }
+    },
+    checkDiscussion : function(page)
+    {
         for (var firstKey in page) break;
         if (page[firstKey] == null) return false;
 
@@ -304,12 +444,13 @@ var Chat = {
             ? firstMessage.jidto
             : firstMessage.jidfrom;
 
-        if (document.getElementById(MovimUtils.cleanupId(contactJid + '-discussion'))
+        if (document.getElementById(MovimUtils.cleanupId(contactJid) + '-discussion')
         == null) return false;
 
         return true;
     },
-    appendMessagesWrapper : function(page, prepend, scroll) {
+    appendMessagesWrapper : function(page, prepend, scroll)
+    {
         var discussion = Chat.getDiscussion();
 
         if (page && Chat.checkDiscussion(page)) {
@@ -363,7 +504,6 @@ var Chat = {
                 );
             }
         } else if (discussion !== null) {
-            let messages = document.querySelector('#chat_widget .contained');
             if (discussion.querySelector('ul').innerHTML === '') {
                 discussion.querySelector('ul').classList.remove('spin');
                 discussion.querySelector('.placeholder').classList.add('show');
@@ -371,6 +511,8 @@ var Chat = {
         }
 
         Chat.setScrollBehaviour();
+        Chat.setReactionButtonBehaviour();
+        Chat.setActionsButtonBehaviour();
 
         if (scroll) {
             MovimTpl.scrollPanel();
@@ -388,10 +530,10 @@ var Chat = {
         var jidtime = idjidtime.substring(idjidtime.indexOf('<') + 1);
 
         if (prepend) {
-            refBubble = document.querySelector('#chat_widget .contained li:first-child');
+            refBubble = document.querySelector('#chat_widget .contained section > ul > li:first-child');
             msgStack = document.querySelector("[data-bubble='" + jidtime + "']");
         } else {
-            refBubble = document.querySelector("#chat_widget .contained li:last-child");
+            refBubble = document.querySelector("#chat_widget .contained section > ul > li:last-child");
             var stack = document.querySelectorAll("[data-bubble='" + jidtime + "']");
             msgStack = stack[stack.length-1];
         }
@@ -402,6 +544,7 @@ var Chat = {
             && (data.sticker === undefined || data.sticker === null)
             && !refBubble.querySelector('div.bubble').classList.contains('sticker')
             && !refBubble.querySelector('div.bubble').classList.contains('file')
+            && ['jingle_start'].indexOf(data.type) < 0
         ) {
             bubble = msgStack.parentNode;
             mergeMsg = true;
@@ -410,44 +553,67 @@ var Chat = {
             || data.mine) {
                 bubble = Chat.right.cloneNode(true);
                 if (data.mine) {
-                    id = data.jidfrom + '_conversation';
+                    id = data.jidfrom;
                 } else {
-                    id = data.jidto + '_conversation';
+                    id = data.jidto;
                 }
             } else {
                 bubble = Chat.left.cloneNode(true);
-                id = data.jidfrom + '_conversation';
+                id = data.jidfrom;
             }
 
-            id = MovimUtils.cleanupId(id);
+            id = MovimUtils.cleanupId(id) + '-conversation';
 
             bubble.querySelector('div.bubble').dataset.bubble = jidtime;
             bubble.querySelector('div.bubble').dataset.publishedprepared = data.publishedPrepared;
         }
 
+        if (['jingle_start'].indexOf(data.type) >= 0) {
+            bubble.querySelector('div.bubble').classList.add('call');
+        }
+
         var msg = bubble.querySelector('div.bubble > div');
-        var span = msg.getElementsByTagName('span')[0];
+        var span = msg.querySelector('span:not(.reaction)');
         var p = msg.getElementsByTagName('p')[0];
+        var reaction = msg.querySelector('span.reaction');
+        var actions = msg.querySelector('span.actions');
+        var reactions = msg.querySelector('ul.reactions');
 
         // If there is already a msg in this bubble, create another div (next msg or replacement)
         if (bubble.querySelector('div.bubble p')
         && bubble.querySelector('div.bubble p').innerHTML != '') {
             msg = document.createElement('div');
-            p = document.createElement('p');
             span = document.createElement('span');
             span.className = 'info';
+            p = document.createElement('p');
+            reaction = reaction.cloneNode(true);
+
+            if (actions) {
+                actions = actions.cloneNode(true);
+            }
+
+            reactions = document.createElement('ul');
+            reactions.className = 'reactions';
         }
 
         if (data.rtl) {
             bubble.querySelector('div.bubble').setAttribute('dir', 'rtl');
         }
 
+        if (data.retracted) {
+            p.classList.add('retracted');
+        }
+
+        if (data.encrypted) {
+            p.classList.add('encrypted');
+        }
+
         if (data.body.match(/^\/me\s/)) {
             p.classList.add('quote');
             data.body = data.body.substr(4);
         }
 
-        if (data.body.match(/^\/code/)) {
+        if (data.body.match(/^\/code\s/)) {
             p.classList.add('code');
             data.body = data.body.substr(6).trim();
         }
@@ -459,12 +625,22 @@ var Chat = {
         if (data.sticker != null) {
             bubble.querySelector('div.bubble').classList.add('sticker');
             p.appendChild(Chat.getStickerHtml(data.sticker));
+
+            if (data.file != null) {
+                p.classList.add('previewable');
+            }
         } else {
             p.innerHTML = data.body;
         }
 
-        if (data.file != null) {
+        if (data.file != null && data.card === undefined && data.file.type !== 'xmpp') {
             bubble.querySelector('div.bubble').classList.add('file');
+
+            // Ugly fix to clear the paragraph if the file contains a similar link
+            if (p.querySelector('a') && p.querySelector('a').href == data.file.uri) {
+                p.innerHTML = '';
+            }
+
             p.appendChild(Chat.getFileHtml(data.file, data.sticker));
         }
 
@@ -480,8 +656,26 @@ var Chat = {
             }
         }
 
+        if (data.reactionsHtml !== undefined) {
+            reactions.innerHTML = data.reactionsHtml;
+        }
+
+        if (data.card) {
+            bubble.querySelector('div.bubble').classList.add('file');
+            msg.appendChild(Chat.getCardHtml(data.card));
+        }
+
         msg.appendChild(p);
         msg.appendChild(span);
+        msg.appendChild(reactions);
+
+        reaction.dataset.mid = data.mid;
+        msg.appendChild(reaction);
+
+        if (actions) {
+            actions.dataset.mid = data.mid;
+            msg.appendChild(actions);
+        }
 
         var elem = document.getElementById(data.oldid);
         if (!elem) {
@@ -502,7 +696,7 @@ var Chat = {
         /* MUC specific */
         if (isMuc) {
             bubble.querySelector('div.bubble').dataset.publishedprepared =
-                data.resource + ' – ' + data.publishedPrepared;
+                data.resource + ' · ' + data.publishedPrepared;
 
             if (data.moderator) {
                 bubble.querySelector('div.bubble').classList.add('moderator');
@@ -581,7 +775,7 @@ var Chat = {
 
         var list = document.querySelector('#chat_widget > div ul');
 
-        if (list.querySelector('li.separator')) return;
+        if (!list || list.querySelector('li.separator')) return;
 
         var messages = document.querySelectorAll('#chat_widget > div ul div.bubble p');
 
@@ -607,32 +801,49 @@ var Chat = {
             }
         }
 
+        if (sticker.title) {
+            img.title = sticker.title;
+        }
+
         if (sticker.picture) {
             img.classList.add('active');
+            img.classList.add('transparent');
             img.setAttribute('onclick', 'Preview_ajaxShow("' + sticker.url + '")');
         }
 
         return img;
     },
+    getCardHtml: function(card) {
+        var ul = document.createElement('ul');
+        ul.setAttribute('class', 'card list noanim active');
+
+        ul.innerHTML = card;
+        return ul;
+    },
     getFileHtml: function(file, sticker) {
         var div = document.createElement('div');
         div.setAttribute('class', 'file');
 
-        var a = document.createElement('a');
-        if (sticker == null) {
-            a.innerHTML = file.name;
+        if (file.name) {
+            var a = document.createElement('a');
+
+            if (sticker == null) {
+                var link = document.createElement('p');
+                link.textContent = file.name;
+                a.appendChild(link);
+            }
+            a.setAttribute('href', file.uri);
+            a.setAttribute('target', '_blank');
+            a.setAttribute('rel', 'noopener');
+
+            div.appendChild(a);
+
+            var span = document.createElement('span');
+            span.innerHTML = file.cleansize;
+            span.setAttribute('class', 'size');
+
+            a.appendChild(span);
         }
-        a.setAttribute('href', file.uri);
-        a.setAttribute('target', '_blank');
-        a.setAttribute('rel', 'noopener');
-
-        div.appendChild(a);
-
-        var span = document.createElement('span');
-        span.innerHTML = file.cleansize;
-        span.setAttribute('class', 'size');
-
-        a.appendChild(span);
 
         return div;
     },
@@ -657,15 +868,26 @@ var Chat = {
         return i;
     },
     toggleAction: function() {
-        var send_button = document.querySelector('.chat_box span.send');
-        var attachment_button = document.querySelector('.chat_box span.upload');
-        if (send_button && attachment_button) {
+        var chatBox = document.querySelector('#chat_widget .chat_box');
+
+        if (chatBox) {
             if (Chat.getTextarea().value.length > 0) {
-                MovimUtils.showElement(send_button);
-                MovimUtils.hideElement(attachment_button);
+                chatBox.classList.add('compose');
+                Chat.toggleAttach(true);
             } else {
-                MovimUtils.showElement(attachment_button);
-                MovimUtils.hideElement(send_button);
+                chatBox.classList.remove('compose');
+            }
+        }
+    },
+    toggleAttach: function(forceDisabled)
+    {
+        var attach = document.querySelector('#chat_widget .chat_box span.attach');
+
+        if (attach) {
+            if (forceDisabled) {
+                attach.classList.remove('enabled');
+            } else {
+                attach.classList.toggle('enabled');
             }
         }
     },
@@ -675,6 +897,50 @@ var Chat = {
     },
     getDiscussion: function() {
         return document.querySelector('#chat_widget div.contained');
+    },
+    touchEvents: function() {
+        var chat = document.querySelector('#chat_widget');
+        clientWidth = Math.abs(document.body.clientWidth);
+
+        chat.addEventListener('touchstart', function(event) {
+            Chat.startX = event.targetTouches[0].pageX;
+            Chat.startY = event.targetTouches[0].pageY;
+            chat.classList.remove('moving');
+        }, true);
+
+        chat.addEventListener('touchmove', function(event) {
+            moveX = event.targetTouches[0].pageX;
+            moveY = event.targetTouches[0].pageY;
+            delay = 20;
+            Chat.translateX = parseInt(moveX - Chat.startX);
+            Chat.translateY = parseInt(moveY - Chat.startY);
+
+            if (Chat.translateX > delay && Chat.translateX <= clientWidth) {
+                // If the horizontal movement is allowed and the vertical one is not important
+                // we authorize the slide
+                if (Chat.translateY < delay) {
+                    Chat.slideAuthorized = true;
+                }
+
+                if (Chat.slideAuthorized) {
+                    chat.style.touchAction = 'none';
+                    chat.style.transform = 'matrix3d(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, '
+                        + (Chat.translateX - delay)
+                        + ', 0, 0, 1)';
+                }
+            }
+        }, true);
+
+        chat.addEventListener('touchend', function(event) {
+            chat.classList.add('moving');
+            if (Chat.translateX > (clientWidth / 4) && Chat.slideAuthorized) {
+                MovimTpl.hidePanel();
+                Chat_ajaxGet(null, true);
+            }
+            chat.style.transform = '';
+            Chat.slideAuthorized = false;
+            Chat.startX = Chat.translateX = Chat.startY = Chat.translateY = 0;
+        }, true);
     }
 };
 
@@ -687,20 +953,20 @@ MovimWebsocket.attach(function() {
         if (Boolean(document.getElementById(MovimUtils.cleanupId(jid) + '-conversation'))) {
             Chat_ajaxGetHistory(jid, Chat.currentDate, room, false);
         } else {
-            MovimTpl.showPanel();
-
             if (room) {
                 Chat_ajaxGetRoom(jid);
             } else {
                 Chat_ajaxGet(jid);
             }
         }
+    } else {
+        Notification.current('chat');
     }
 });
 
 if (typeof Upload != 'undefined') {
     Upload.attach(function(file) {
-        Chat_ajaxHttpSendMessage(Chat.getTextarea().dataset.jid, false, Boolean(Chat.getTextarea().dataset.muc), false, false, file);
+        Chat_ajaxHttpDaemonSendMessage(Chat.getTextarea().dataset.jid, false, Boolean(Chat.getTextarea().dataset.muc), false, false, file);
     });
 }
 
@@ -717,4 +983,8 @@ window.addEventListener('resize', function() {
     }
 });
 
+movim_add_onload(function() {
+    Chat.touchEvents();
+});
+
 var state = 0;
