@@ -1,4 +1,5 @@
 MovimWebsocket.attach(function() {
     Notification.current('community');
-    Communities_ajaxGet();
 });
+
+MovimWebsocket.initiate(() => Communities_ajaxHttpGet());
