@@ -7,7 +7,4 @@ var Presence = {
     }
 }
 
-MovimWebsocket.attach(function()
-{
-    Presence_ajaxGetPresence();
-});
+MovimWebsocket.initiate(() => Presence_ajaxHttpGetPresence());
