@@ -1,13 +1,14 @@
 <?php
 
 use Moxl\Xec\Action\Presence\Muc;
-use Moxl\Xec\Action\Bookmark\Get;
-use Moxl\Xec\Action\Bookmark\Set;
+use Moxl\Xec\Action\Bookmark2\Set;
+use Moxl\Xec\Action\Bookmark2\Delete;
 use Moxl\Xec\Action\Presence\Unavailable;
 use Moxl\Xec\Action\Message\Invite;
 use Moxl\Xec\Action\Disco\Request;
 use Moxl\Xec\Action\Disco\Items;
 use Moxl\Xec\Action\Muc\SetSubject;
+use Moxl\Xec\Action\Muc\Destroy;
 use Moxl\Xec\Action\Vcard\Set as VcardSet;
 
 use Respect\Validation\Validator;
@@ -18,6 +19,8 @@ use Movim\Picture;
 use Movim\Session;
 use Movim\ChatStates;
 
+use App\Conference;
+
 class Rooms extends Base
 {
     public function load()
@@ -25,40 +28,42 @@ class Rooms extends Base
         $this->addjs('rooms.js');
         $this->addcss('rooms.css');
         $this->registerEvent('message', 'onMessage');
-        $this->registerEvent('bookmark_get_handle', 'onGetBookmark');
-        $this->registerEvent('bookmark_set_handle', 'onBookmark');
+        $this->registerEvent('bookmark2_get_handle', 'onBookmarkGet');
+        $this->registerEvent('bookmark2', 'onBookmarkSet');
+        $this->registerEvent('bookmark2_set_handle', 'onBookmarkSet');
+        $this->registerEvent('bookmark2_delete_handle', 'onBookmarkSet');
+        $this->registerEvent('bookmark_synchronize_handle', 'onBookmarkSynchronized');
         $this->registerEvent('disco_items_nosave_handle', 'onDiscoGateway');
         $this->registerEvent('disco_items_nosave_error', 'onDiscoGatewayError');
         $this->registerEvent('vcard_set_handle', 'onAvatarSet', 'chat');
+        $this->registerEvent('muc_destroy_handle', 'onDestroyed', 'chat');
         $this->registerEvent('presence_muc_handle', 'onConnected', 'chat');
         $this->registerEvent('presence_unavailable_handle', 'onDisconnected', 'chat');
         $this->registerEvent('presence_muc_errorconflict', 'onConflict');
         $this->registerEvent('presence_muc_errorregistrationrequired', 'onRegistrationRequired');
         $this->registerEvent('presence_muc_errorremoteservernotfound', 'onRemoteServerNotFound');
+        $this->registerEvent('presence_muc_erroritemnotfound', 'onItemNotFound');
+        $this->registerEvent('presence_muc_errornotauthorized', 'onNotAuthorized');
         $this->registerEvent('composing', 'onComposing', 'chat');
         $this->registerEvent('paused', 'onPaused', 'chat');
+
+        $this->registerEvent('chat_open_room', 'onChatOpen', 'chat');
     }
 
     public function onMessage($packet)
     {
         $message = $packet->content;
 
-        if ($message->user_id == $message->jidto
-        && $message->type == 'groupchat'
-        && $message->subject == null) {
-            Notification::append(
-                'chat|'.$message->jidfrom,
-                null,
-                $message->body,
-                null,
-                0,
-                null
-            );
-        }
-
         $chatStates = ChatStates::getInstance();
         $chatStates->clearState($message->jidfrom, $message->resource);
         $this->onPaused($chatStates->getState($message->jidfrom));
+
+        $this->setCounter($message->jidfrom);
+    }
+
+    public function onChatOpen(string $room)
+    {
+        $this->setCounter($room);
     }
 
     public function onDiscoGateway($packet)
@@ -86,22 +91,34 @@ class Rooms extends Base
     public function onAvatarSet($packet)
     {
         $this->rpc('Dialog_ajaxClear');
-        Notification::append(null, $this->__('avatar.updated'));
+        Notification::toast($this->__('avatar.updated'));
     }
 
     public function onRegistrationRequired($packet)
     {
-        Notification::append(null, $this->__('chatrooms.registrationrequired'));
+        Notification::toast($this->__('chatrooms.registrationrequired'));
         $this->ajaxExit($packet->content);
     }
 
     public function onRemoteServerNotFound($packet)
     {
-        Notification::append(null, $this->__('chatrooms.remoteservernotfound'));
+        Notification::toast($this->__('chatrooms.remoteservernotfound'));
         $this->ajaxExit($packet->content);
     }
 
-    public function onGetBookmark()
+    public function onItemNotFound($packet)
+    {
+        Notification::toast($this->__('chatrooms.itemnotfound'));
+        $this->ajaxExit($packet->content);
+    }
+
+    public function onNotAuthorized($packet)
+    {
+        Notification::toast($this->__('chatrooms.notauthorized'));
+        $this->ajaxExit($packet->content);
+    }
+
+    public function onBookmarkGet()
     {
         foreach ($this->user->session->conferences as $room) {
             if ($room->autojoin && !$room->connected) {
@@ -112,10 +129,29 @@ class Rooms extends Base
         $this->refreshRooms();
     }
 
-    public function onBookmark()
+    public function onBookmarkSynchronized($packet)
+    {
+        Notification::toast($this->__('chatrooms.synchronized', $packet->content));
+    }
+
+    public function onDestroyed($packet)
     {
         $this->refreshRooms();
-        $this->rpc('MovimTpl.hidePanel');
+        $this->rpc('Chat_ajaxGet');
+
+        Notification::toast($this->__('chatrooms.destroyed'));
+    }
+
+    public function onBookmarkSet($packet)
+    {
+        $conference = $packet->content;
+
+        if ($conference && $conference->autojoin) {
+            $this->ajaxJoin($conference->conference, $conference->nick);
+        }
+
+        Notification::toast($this->__('bookmarks.updated'));
+        $this->refreshRooms();
     }
 
     public function onConnected($packet)
@@ -125,13 +161,13 @@ class Rooms extends Base
 
     public function onConflict()
     {
-        Notification::append(null, $this->__('chatrooms.conflict'));
+        Notification::toast($this->__('chatrooms.conflict'));
     }
 
     public function onDisconnected()
     {
         $this->refreshRooms();
-        Notification::append(null, $this->__('chatrooms.disconnected'));
+        Notification::toast($this->__('chatrooms.disconnected'));
     }
 
     private function setState(string $room, bool $composing)
@@ -141,12 +177,30 @@ class Rooms extends Base
                 ? 'MovimUtils.addClass'
                 : 'MovimUtils.removeClass',
             '#' . cleanupId($room.'_rooms_primary'),
-            'composing');
+            'composing'
+        );
     }
 
-    private function refreshRooms($edit = false)
+    private function setCounter(string $room)
     {
-        $this->rpc('MovimTpl.fill', '#rooms_widget', $this->prepareRooms($edit));
+        $conference = $this->user->session
+                           ->conferences()
+                           ->where('conference', $room)
+                           ->withCount('unreads')
+                           ->first();
+
+        if ($conference) {
+            $this->rpc(
+                'MovimTpl.fill',
+                '#' . cleanupId($room.'_rooms_primary'),
+                $this->prepareRoomCounter($conference, $conference->getPhoto())
+            );
+        }
+    }
+
+    private function refreshRooms($edit = false, $all = false)
+    {
+        $this->rpc('MovimTpl.fill', '#rooms_widget', $this->prepareRooms($edit, $all));
         $this->rpc('Rooms.refresh');
     }
 
@@ -158,37 +212,42 @@ class Rooms extends Base
     /**
      * @brief Get the Rooms
      */
-    public function ajaxDisplay($edit = false)
+    public function ajaxHttpDisplay($edit = false, $all = false)
     {
-        $this->refreshRooms($edit);
+        $this->refreshRooms($edit, $all);
     }
 
     /**
      * @brief Display the add room form
      */
-    public function ajaxAdd($room = false)
+    public function ajaxAdd($room = false, $name = null)
     {
         $view = $this->tpl();
 
         $view->assign('info', \App\Info::where('server', $room)
-                                       ->where('category', 'conference')
+                                       ->where('node', '')
+                                       ->whereCategory('conference')
                                        ->first());
         $view->assign('mucservice', \App\Info::where('server', 'like', '%'. $this->user->session->host)
                                              ->where('server', 'not like', '%@%')
-                                             ->where('category', 'conference')
-                                             ->where('type', 'text')
+                                             ->whereCategory('conference')
+                                             ->whereType('text')
                                              ->first());
         $view->assign('id', $room);
-        $view->assign('conference',
+        $view->assign(
+            'conference',
             $this->user->session->conferences()
-            ->where('conference', $room)->first());
+            ->where('conference', $room)->first()
+        );
+        $view->assign('name', $name);
         $view->assign('username', $this->user->session->username);
-        $view->assign('gateways',
-            \App\Info::whereIn('server', function($query) {
-                            $query->select('jid')->from('presences');
-                        })
-                     ->where('category', 'gateway')
-                     ->get()
+        $view->assign(
+            'gateways',
+            \App\Info::whereIn('server', function ($query) {
+                $query->select('jid')->from('presences');
+            })
+            ->whereCategory('gateway')
+            ->get()
         );
 
         $this->rpc('Rooms.setDefaultServices', $this->user->session->getChatroomsServices());
@@ -196,6 +255,29 @@ class Rooms extends Base
         Dialog::fill($view->draw('_rooms_add'));
     }
 
+    /**
+     * Display the room subject
+     */
+    public function ajaxShowSubject($room = false)
+    {
+        if (!$this->validateRoom($room)) {
+            return;
+        }
+
+        $conference = $this->user->session->conferences()
+            ->where('conference', $room)
+            ->with('info')
+            ->first();
+
+        if (!$conference) return;
+
+        $view = $this->tpl();
+        $view->assign('conference', $conference);
+        $view->assign('room', $room);
+
+        Dialog::fill($view->draw('_rooms_subject_show'));
+    }
+
     /**
      * @brief Discover rooms for a gateway
      */
@@ -217,7 +299,9 @@ class Rooms extends Base
      */
     public function ajaxGetAvatar($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         $view = $this->tpl();
         $view->assign('room', $this->user->session->conferences()
@@ -232,7 +316,9 @@ class Rooms extends Base
      */
     public function ajaxSetAvatar($room, $form)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         $p = new Picture;
         $p->fromBase($form->photobin->value);
@@ -257,7 +343,9 @@ class Rooms extends Base
      */
     public function ajaxGetSubject($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         $view = $this->tpl();
         $view->assign('room', $this->user->session->conferences()
@@ -265,6 +353,7 @@ class Rooms extends Base
                                  ->first());
 
         Dialog::fill($view->draw('_rooms_subject'));
+        $this->rpc('MovimUtils.applyAutoheight');
     }
 
     /**
@@ -283,7 +372,6 @@ class Rooms extends Base
           ->request();
     }
 
-
     /**
      * @brief Display the add room form
      */
@@ -304,7 +392,9 @@ class Rooms extends Base
      */
     public function ajaxInvite($form)
     {
-        if (!$this->validateRoom($form->to->value)) return;
+        if (!$this->validateRoom($form->to->value)) {
+            return;
+        }
 
         if (!empty($form->invite->value)) {
             $i = new Invite;
@@ -313,7 +403,7 @@ class Rooms extends Base
               ->setInvite($form->invite->value)
               ->request();
 
-            Notification::append(null, $this->__('room.invited'));
+            Notification::toast($this->__('room.invited'));
             $this->rpc('Dialog_ajaxClear');
         }
     }
@@ -323,7 +413,9 @@ class Rooms extends Base
      */
     public function ajaxRemoveConfirm($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         $view = $this->tpl();
 
@@ -337,15 +429,21 @@ class Rooms extends Base
      */
     public function ajaxList($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
+
+        $conference = $this->user->session->conferences()
+            ->where('conference', $room)
+            ->first();
+
+        if (!$conference) return;
 
         $view = $this->tpl();
-        $view->assign('list', $this->user->session->conferences()
-                                   ->where('conference', $room)
-                                   ->first()
-                                   ->presences()
-                                   ->with('capability')
-                                   ->get());
+        $view->assign('list', $conference->presences()
+            ->with('capability')
+            ->get());
+        $view->assign('conference', $conference);
         $view->assign('room', $room);
         $view->assign('me', $this->user->id);
 
@@ -368,11 +466,41 @@ class Rooms extends Base
      */
     public function ajaxRemove($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
-        $this->user->session->conferences()->where('conference', $room)->delete();
+        $d = new Delete;
+        $d->setId($room)
+          ->request();
+    }
 
-        $this->setBookmark();
+    /**
+     * @brief Destroy a room
+     */
+    public function ajaxAskDestroy($room)
+    {
+        if (!$this->validateRoom($room)) {
+            return;
+        }
+
+        $view = $this->tpl();
+        $view->assign('room', $room);
+
+        Dialog::fill($view->draw('_rooms_destroy'));
+    }
+    /**
+     * @brief Destroy a room
+     */
+    public function ajaxDestroy($room)
+    {
+        if (!$this->validateRoom($room)) {
+            return;
+        }
+
+        $d = new Destroy;
+        $d->setTo($room)
+          ->request();
     }
 
     /**
@@ -380,7 +508,9 @@ class Rooms extends Base
      */
     public function ajaxJoin($room, $nickname = false)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         $r = new Request;
         $r->setTo($room)
@@ -389,16 +519,14 @@ class Rooms extends Base
         $p = new Muc;
         $p->setTo($room);
 
-        /*$c = new \Moxl\Xec\Action\Disco\Request;
-        $c->setTo(explodeJid($room)['server'])
-          ->request();*/
-
         if ($nickname == false) {
             $nickname = $this->user->session->username;
         }
 
         $jid = explodeJid($room);
-        $capability = App\Capability::find($jid['server']);
+        $capability = \App\Info::where('server', $jid['server'])
+                               ->where('node', '')
+                               ->first();
 
         if ($capability && ($capability->isMAM() || $capability->isMAM2())) {
             $p->enableMAM();
@@ -423,7 +551,9 @@ class Rooms extends Base
      */
     public function ajaxExit($room)
     {
-        if (!$this->validateRoom($room)) return;
+        if (!$this->validateRoom($room)) {
+            return;
+        }
 
         // We reset the Chat view
         $this->rpc('Chat_ajaxGet');
@@ -431,13 +561,23 @@ class Rooms extends Base
         // We properly exit
         $resource = $this->user->session->conferences()
             ->where('conference', $room)
-            ->first()->presences()
+            ->first();
+
+        if (!$resource) return;
+
+        $resource = $resource
+            ->presences()
             ->where('mucjid', $this->user->id)
-            ->first()
-            ->resource;
+            ->first();
+
+        $resource = $resource
+            ? $resource->resource
+            : null;
 
         $jid = explodeJid($room);
-        $capability = App\Capability::find($jid['server']);
+        $capability = \App\Info::where('server', $jid['server'])
+                               ->where('node', '')
+                               ->first();
 
         if (!$capability || !$capability->isMAM()) {
             $this->user->messages()->where('jidfrom', $room)->delete();
@@ -449,13 +589,15 @@ class Rooms extends Base
 
         $this->refreshRooms();
 
-        $session = Session::start();
-        $session->remove($room . '/' .$resource);
+        if ($resource) {
+            $session = Session::start();
+            $session->remove($room . '/' .$resource);
 
-        $pu = new Unavailable;
-        $pu->setTo($room)
-           ->setResource($resource)
-           ->request();
+            $pu = new Unavailable;
+            $pu->setTo($room)
+               ->setResource($resource)
+               ->request();
+        }
     }
 
     /**
@@ -463,80 +605,94 @@ class Rooms extends Base
      */
     public function ajaxChatroomAdd($form)
     {
-        if (!filter_var($form->jid->value, FILTER_VALIDATE_EMAIL)) {
-            Notification::append(null, $this->__('chatrooms.bad_id'));
+        if (!$this->validateRoom($form->jid->value)) {
+            Notification::toast($this->__('chatrooms.bad_id'));
         } elseif (trim($form->name->value) == '') {
-            Notification::append(null, $this->__('chatrooms.empty_name'));
+            Notification::toast($this->__('chatrooms.empty_name'));
         } else {
+            $this->ajaxExit($form->jid->value);
+
             $this->user->session->conferences()
                  ->where('conference', strtolower($form->jid->value))
                  ->delete();
 
-            $item = [
-                    'type'      => 'conference',
-                    'name'      => $form->name->value,
-                    'autojoin'  => $form->autojoin->value,
-                    'nick'      => $form->nick->value,
-                    'jid'       => strtolower($form->jid->value)
-                    ];
-            $this->setBookmark($item);
+            $conference = new Conference;
+            $conference->conference = strtolower($form->jid->value);
+            $conference->name = $form->name->value;
+            $conference->autojoin = $form->autojoin->value;
+            $conference->nick = $form->nick->value;
+
+            $b = new Set;
+            $b->setConference($conference)
+              ->request();
+
             $this->rpc('Dialog_ajaxClear');
         }
     }
 
-    public function setBookmark($item = false)
+    /**
+     * Synchronize Bookmark 1 to Bookmark 2
+     */
+    public function ajaxSyncBookmark()
     {
-        $arr = [];
-
-        if ($item) {
-            array_push($arr, $item);
-        }
-
-        $conferences = $this->user->session->conferences()->get();
-        if ($conferences) {
-            foreach ($conferences as $c) {
-                array_push($arr,
-                    [
-                        'type'      => 'conference',
-                        'name'      => $c->name,
-                        'autojoin'  => $c->autojoin,
-                        'nick'      => $c->nick,
-                        'jid'       => $c->conference
-                    ]
-                );
-            }
-        }
-
-        $b = new Set;
-        $b->setArr($arr)
-          ->request();
+        $s = new \Moxl\Xec\Action\Bookmark\Synchronize;
+        $s->request();
     }
 
-    public function prepareRooms($edit = false)
+    public function prepareRooms($edit = false, $all = false)
     {
-        if (!$this->user->session) return '';
+        if (!$this->user->session) {
+            return '';
+        }
 
         $conferences = $this->user->session->conferences()
                                            ->with('info', 'contact', 'presence')
+                                           ->withCount('unreads')
                                            ->get();
         $connected = new Collection;
+        $disconnected = 0;
+        $servers = [];
 
         foreach ($conferences as $key => $conference) {
+            array_push($servers, $conference->server);
             if ($conference->connected) {
                 $connected->push($conferences->pull($key));
+            } else {
+                $disconnected++;
             }
         }
 
         $conferences = $connected->merge($conferences);
 
+        // If all the rooms are disconnected, we show everything
+        if ($disconnected == $conferences->count()) {
+            $all = true;
+        }
+
         $view = $this->tpl();
         $view->assign('edit', $edit);
+        $view->assign('all', $all);
+        $view->assign('disconnected', $disconnected);
+        $view->assign('servers', App\Info::with('identities')
+                                         ->whereIn('server', array_unique($servers))
+                                         ->get()
+                                         ->keyBy('server')
+        );
         $view->assign('conferences', $conferences);
         $view->assign('room', $this->get('r'));
 
         return $view->draw('_rooms');
     }
 
+    public function prepareRoomCounter(Conference $conference, $withAvatar = false)
+    {
+        $view = $this->tpl();
+        $view->assign('conference', $conference);
+        $view->assign('withAvatar', $withAvatar);
+
+        return $view->draw('_rooms_counter');
+    }
+
     /**
      * @brief Validate the room
      *
@@ -544,16 +700,6 @@ class Rooms extends Base
      */
     private function validateRoom($room)
     {
-        return (Validator::stringType()->noWhitespace()->length(6, 80)->validate($room));
-    }
-
-    /**
-     * @brief Validate the resource
-     *
-     * @param string $resource
-     */
-    private function validateResource($resource)
-    {
-        return (Validator::stringType()->length(2, 40)->validate($resource));
+        return (Validator::stringType()->noWhitespace()->contains('@')->length(6, 256)->validate($room));
     }
 }
