@@ -11,7 +11,7 @@ class Statistics extends Base
 
     public function getCapabilityName($node)
     {
-        $capability = App\Capability::where('node', 'like', '%' . $node . '%')->first();
+        $capability = \App\Info::where('node', 'like', '%' . $node . '%')->first();
 
         if ($capability && !filter_var($capability->name, FILTER_VALIDATE_URL)) {
             $parts = explode(' ', $capability->name);
@@ -36,7 +36,9 @@ class Statistics extends Base
             list($client, $version) = explode('#', $presence->node);
             $parts = explode('/', $client);
             $part = isset($parts[2]) ? $parts[2] : $client;
-            if (!isset($stats[$part])) $stats[$part] = 0;
+            if (!isset($stats[$part])) {
+                $stats[$part] = 0;
+            }
 
             $stats[$part]++;
             $total++;
