@@ -7,6 +7,7 @@ require dirname(__FILE__) . '/vendor/autoload.php';
 use Movim\Bootstrap;
 use Movim\Console\DaemonCommand;
 use Movim\Console\ConfigCommand;
+use Movim\Console\EmojisToJsonCommand;
 use Symfony\Component\Console\Application;
 
 $bootstrap = new Bootstrap;
@@ -15,5 +16,5 @@ $bootstrap->boot($argv[1] == 'help');
 $application = new Application;
 $application->add(new ConfigCommand);
 $application->add(new DaemonCommand);
+$application->add(new EmojisToJsonCommand);
 $application->run();
-
