@@ -13,8 +13,8 @@ class SDPtoJingle
 
     private $action;
 
-    private $mid;
-    private $mlineindex;
+    private $mid = null;
+    private $sid;
 
     // Move the global fingerprint into each medias
     private $global_fingerprint = [];
@@ -22,7 +22,7 @@ class SDPtoJingle
     private $rtcp_fb_cache = [];
 
     private $regex = [
-      'candidate'       => "/^a=candidate:(\w{1,32}) (\d{1,5}) (udp|tcp) (\d{1,10}) ([a-zA-Z0-9:\.]{1,45}) (\d{1,5}) (typ) (host|srflx|prflx|relay)( (raddr) ([a-zA-Z0-9:\.]{1,45}) (rport) (\d{1,5}))?( (generation) (\d) (network) (\d) (id) ([a-zA-Z0-9]{1,45}))?/i", //à partir de generation les attr sont spécifiques à XMPP..autant l'enlever de la REGEX et les traiter à part? En théorie ils peuvent être dans n'importe quel ordre.
+      'candidate'       => "/^a=candidate:(\w{1,32}) (\d{1,5}) (udp|tcp) (\d{1,10}) ([a-zA-Z0-9:\.]{1,45}) (\d{1,5}) (typ) (host|srflx|prflx|relay)\s?(.+)?/i",
       'sess_id'         => "/^o=(\S+) (\d+)/i",
       'group'           => "/^a=group:(\S+) (.+)/i",
       'rtpmap'          => "/^a=rtpmap:(\d+) (([^\s\/]+)(\/(\d+)(\/([^\s\/]+))?)?)?/i",
@@ -46,24 +46,34 @@ class SDPtoJingle
       'media'           => "/^m=(audio|video|application|data)/i"
     ];
 
-    public function __construct($sdp, $initiator, $responder = false, $action = false, $mid = false, $mlineindex = false)
+    public function __construct($sdp, $initiator, $responder = false, $action = false, $mid = null)
     {
         $this->sdp = $sdp;
         $this->arr = explode("\n", $this->sdp);
 
-        if ($mid) $this->mid = $mid;
-        if ($mlineindex) $this->mlineindex = $mlineindex;
+        if ($mid) {
+            $this->mid = $mid;
+        }
 
         $this->jingle = new SimpleXMLElement('<jingle></jingle>');
         $this->jingle->addAttribute('xmlns', 'urn:xmpp:jingle:1');
-        $this->jingle->addAttribute('initiator',$initiator);
+        $this->jingle->addAttribute('initiator', $initiator);
 
-        if ($action) $this->jingle->addAttribute('action',$action);
-        if ($responder) $this->jingle->addAttribute('responder',$responder);
+        if ($action) {
+            $this->jingle->addAttribute('action', $action);
+        }
+        if ($responder) {
+            $this->jingle->addAttribute('responder', $responder);
+        }
 
         $this->action = $action;
     }
 
+    public function setSessionId(string $sid)
+    {
+        $this->sid = $sid;
+    }
+
     private function getSessionId()
     {
         $s = Session::start();
@@ -109,20 +119,20 @@ class SDPtoJingle
         foreach ($params as $matches) {
             $rtcpfp = $payloadtype->addChild('rtcp-fb');
             $rtcpfp->addAttribute('xmlns', "urn:xmpp:jingle:apps:rtp:rtcp-fb:0");
-            $rtcpfp->addAttribute('id',        $matches[1]);
-            $rtcpfp->addAttribute('type',      $matches[2]);
+            $rtcpfp->addAttribute('id', $matches[1]);
+            $rtcpfp->addAttribute('type', $matches[2]);
 
             if (isset($matches[4])) {
-                $rtcpfp->addAttribute('subtype',   $matches[4]);
+                $rtcpfp->addAttribute('subtype', $matches[4]);
             }
         }
     }
 
-    public function addName($name = false)
+    public function addName($name = null)
     {
-        if ($name) {
+        if ($name !== null) {
             $this->content->addAttribute('name', $name);
-        } elseif ($this->mid) {
+        } elseif ($this->mid !== null) {
             $this->content->addAttribute('name', $this->mid);
         }
     }
@@ -132,9 +142,9 @@ class SDPtoJingle
         foreach ($this->arr as $l) {
             foreach ($this->regex as $key => $r) {
                 if (preg_match($r, $l, $matches)) {
-                    switch($key) {
+                    switch ($key) {
                         case 'sess_id':
-                            $this->jingle->addAttribute('sid', $this->getSessionId());
+                            $this->jingle->addAttribute('sid', $this->sid ?? $this->getSessionId());
                             break;
                         case 'media':
                             $this->initContent(true);
@@ -162,19 +172,21 @@ class SDPtoJingle
 
                         case 'bandwidth':
                             $bandwidth = $description->addChild('bandwidth');
-                            $bandwidth->addAttribute('type',       $matches[1]);
-                            $bandwidth->addAttribute('value',      $matches[2]);
+                            $bandwidth->addAttribute('type', $matches[1]);
+                            $bandwidth->addAttribute('value', $matches[2]);
                             break;
 
                         case 'rtpmap':
                             $payloadtype = $description->addChild('payload-type');
-                            $payloadtype->addAttribute('id',        $matches[1]);
-                            $payloadtype->addAttribute('name',      $matches[3]);
-                            if (isset($matches[4]))
+                            $payloadtype->addAttribute('id', $matches[1]);
+                            $payloadtype->addAttribute('name', $matches[3]);
+                            if (isset($matches[4])) {
                                 $payloadtype->addAttribute('clockrate', $matches[5]);
+                            }
 
-                            if (isset($matches[7]))
-                                $payloadtype->addAttribute('channels',   $matches[7]);
+                            if (isset($matches[7])) {
+                                $payloadtype->addAttribute('channels', $matches[7]);
+                            }
 
                             if (isset($this->fmtp_cache[$matches[1]])) {
                                 $this->addFmtpParameters($payloadtype, $this->fmtp_cache[$matches[1]]);
@@ -225,26 +237,27 @@ class SDPtoJingle
                         case 'rtcp_fb_trr_int':
                             $rtcpfp = $payloadtype->addChild('rtcp-fb-trr-int');
                             $rtcpfp->addAttribute('xmlns', "urn:xmpp:jingle:apps:rtp:rtcp-fb:0");
-                            $rtcpfp->addAttribute('id',        $matches[1]);
-                            $rtcpfp->addAttribute('value',     $matches[2]);
+                            $rtcpfp->addAttribute('id', $matches[1]);
+                            $rtcpfp->addAttribute('value', $matches[2]);
                             break;
 
                         // http://xmpp.org/extensions/xep-0167.html#srtp
                         case 'crypto':
                             $encryption = $description->addChild('encryption');
                             $crypto     = $encryption->addChild('crypto');
-                            $crypto->addAttribute('crypto-suite',   $matches[2]);
-                            $crypto->addAttribute('key-params',     $matches[3]);
-                            $crypto->addAttribute('tag',            $matches[1]);
-                            if (isset($matches[5]))
+                            $crypto->addAttribute('crypto-suite', $matches[2]);
+                            $crypto->addAttribute('key-params', $matches[3]);
+                            $crypto->addAttribute('tag', $matches[1]);
+                            if (isset($matches[5])) {
                                 $crypto->addAttribute('session-params', $matches[5]);
+                            }
                             break;
 
                         // http://xmpp.org/extensions/xep-0262.html
                         case 'zrtp_hash':
                             $zrtphash   = $encryption->addChild('zrtp-hash', $matches[2]);
-                            $zrtphash->addAttribute('xmlns',   "urn:xmpp:jingle:apps:rtp:zrtp:1");
-                            $zrtphash->addAttribute('version',   $matches[1]);
+                            $zrtphash->addAttribute('xmlns', "urn:xmpp:jingle:apps:rtp:zrtp:1");
+                            $zrtphash->addAttribute('version', $matches[1]);
                             break;
 
                         case 'rtcp_mux':
@@ -254,24 +267,25 @@ class SDPtoJingle
                         // http://xmpp.org/extensions/xep-0294.html
                         case 'extmap':
                             $rtphdrext = $description->addChild('rtp-hdrext');
-                            $rtphdrext->addAttribute('xmlns',   "urn:xmpp:jingle:apps:rtp:rtp-hdrext:0");
-                            $rtphdrext->addAttribute('id',      $matches[1]);
-                            $rtphdrext->addAttribute('uri',     $matches[4]);
-                            if (isset($matches[3]) && $matches[3] != '')
+                            $rtphdrext->addAttribute('xmlns', "urn:xmpp:jingle:apps:rtp:rtp-hdrext:0");
+                            $rtphdrext->addAttribute('id', $matches[1]);
+                            $rtphdrext->addAttribute('uri', $matches[4]);
+                            if (isset($matches[3]) && $matches[3] != '') {
                                 $rtphdrext->addAttribute('senders', $matches[3]);
+                            }
                             break;
 
                         // http://xmpp.org/extensions/xep-0339.html
                         case 'ssrc':
                             if (!$description->source) {
                                 $ssrc = $description->addChild('source');
-                                $ssrc->addAttribute('xmlns',   "urn:xmpp:jingle:apps:rtp:ssma:0");
-                                $ssrc->addAttribute('id',   $matches[1]);
+                                $ssrc->addAttribute('xmlns', "urn:xmpp:jingle:apps:rtp:ssma:0");
+                                $ssrc->addAttribute('id', $matches[1]);
                             }
 
                             $param = $ssrc->addChild('parameter');
-                            $param->addAttribute('name',   $matches[2]);
-                            $param->addAttribute('value',  $matches[4]);
+                            $param->addAttribute('name', $matches[2]);
+                            $param->addAttribute('value', $matches[4]);
                             break;
 
                         case 'ptime':
@@ -285,7 +299,7 @@ class SDPtoJingle
                         // http://xmpp.org/extensions/xep-0338.html
                         case 'group':
                             $group = $this->jingle->addChild('group');
-                            $group->addAttribute('xmlns',   "urn:xmpp:jingle:apps:grouping:0");
+                            $group->addAttribute('xmlns', "urn:xmpp:jingle:apps:grouping:0");
                             $group->addAttribute('semantics', $matches[1]);
 
                             $params = explode(' ', $matches[2]);
@@ -340,48 +354,55 @@ class SDPtoJingle
                             $this->initContent();
                             $this->addName();
 
-                            $generation = $network = $id = $networkid = false;
-
-                            if ($key = array_search("generation", $matches))
-                                $generation = $matches[($key+1)];
-                            if ($key = array_search("network", $matches))
-                                $network = $matches[($key+1)];
-                            if ($key = array_search("id", $matches))
-                                $id = $matches[($key+1)];
-                            if ($key = array_search("network-id", $matches))
-                                $networkid = $matches[($key+1)];
-
-                            if (isset($matches[11]) && isset($matches[13])) {
-                                $reladdr = $matches[11];
-                                $relport = $matches[13];
-                            } else {
-                                $reladdr = $relport = null;
-                            }
-
                             $candidate = $this->transport->addChild('candidate');
 
-                            $candidate->addAttribute('component' , $matches[2]);
                             $candidate->addAttribute('foundation', $matches[1]);
+                            $candidate->addAttribute('component', $matches[2]);
+                            $candidate->addAttribute('protocol', $matches[3]);
+                            $candidate->addAttribute('priority', $matches[4]);
+                            $candidate->addAttribute('ip', $matches[5]);
+                            $candidate->addAttribute('port', $matches[6]);
+                            $candidate->addAttribute('type', $matches[8]);
 
-                            if ($generation)
-                                $candidate->addAttribute('generation', $generation);
-                            if ($id)
-                                $candidate->addAttribute('id'        , $id);
-                            if ($network)
-                                $candidate->addAttribute('network'   , $network);
-                            if ($networkid)
-                                $candidate->addAttribute('network-id', $networkid);
+                            // We have other arguments
+                            $args = [];
+                            if (isset($matches[9])) {
+                                $keyValues = explode(' ', $matches[9]);
+                                foreach ($keyValues as $key)
 
-                            $candidate->addAttribute('ip'        , $matches[5]);
-                            $candidate->addAttribute('port'      , $matches[6]);
-                            $candidate->addAttribute('priority'  , $matches[4]);
-                            $candidate->addAttribute('protocol'  , $matches[3]);
-                            $candidate->addAttribute('type'      , $matches[8]);
-
-                            if ($reladdr) {
-                                $candidate->addAttribute('rel-addr'  , $reladdr);
-                                $candidate->addAttribute('rel-port'  , $relport);
+                                foreach (array_chunk($keyValues, 2) as $pair) {
+                                    list($key, $value) = $pair;
+                                    $args[$key] = $value;
+                                }
                             }
+
+                            if (isset($args['generation'])) {
+                                $candidate->addAttribute('generation', $args['generation']);
+                            }
+                            if (isset($args['id'])) {
+                                $candidate->addAttribute('id', $args['id']);
+                            }
+                            if (isset($args['network'])) {
+                                $candidate->addAttribute('network', $args['network']);
+                            }
+                            if (isset($args['network-id'])) {
+                                $candidate->addAttribute('network-id', $args['network-id']);
+                            }
+                            if (isset($args['network-cost'])) {
+                                $candidate->addAttribute('network-cost', $args['network-cost']);
+                            }
+                            if (isset($args['raddr'])) {
+                                $candidate->addAttribute('rel-addr', $args['raddr']);
+                            }
+                            if (isset($args['rport'])) {
+                                $candidate->addAttribute('rel-port', $args['rport']);
+                            }
+
+                            // ufrag to the transport
+                            if (isset($args['ufrag'])) {
+                                $this->transport->addAttribute('ufrag', $args['ufrag']);
+                            }
+
                             break;
                     }
                 }
