@@ -6,7 +6,7 @@ namespace Moxl\Stanza;
 
 class Carbons
 {
-    static function enable()
+    public static function enable()
     {
         $dom = new \DOMDocument('1.0', 'UTF-8');
         $xml = $dom->createElementNS('urn:xmpp:carbons:2', 'enable');
@@ -15,7 +15,7 @@ class Carbons
         \Moxl\API::request($xml);
     }
 
-    static function disable()
+    public static function disable()
     {
         $dom = new \DOMDocument('1.0', 'UTF-8');
         $xml = $dom->createElementNS('urn:xmpp:carbons:2', 'disable');
