@@ -6,7 +6,7 @@ use Movim\Session;
 
 class Ping
 {
-    static function server()
+    public static function server()
     {
         $session = Session::start();
         $dom = new \DOMDocument('1.0', 'UTF-8');
@@ -14,7 +14,7 @@ class Ping
         \Moxl\API::request(\Moxl\API::iqWrapper($ping, $session->get('host'), 'get'));
     }
 
-    static function pong($to, $id)
+    public static function pong($to, $id)
     {
         $dom = new \DOMDocument('1.0', 'UTF-8');
         $ping = $dom->createElementNS('urn:xmpp:ping', 'ping');
