@@ -12,20 +12,19 @@ class Publish extends Action
     protected $_content;
     protected $_html;
     protected $_muc = false;
-    protected $_encrypted = false;
     protected $_id = false;
     protected $_replace = false;
     protected $_file = false;
+    protected $_attachid = false;
+    protected $_originid = false;
 
     public function request()
     {
-        $this->store();
+        $this->store($this->_id);
         if ($this->_muc) {
-            Muc::message($this->_to, $this->_content, $this->_html, $this->_id, $this->_file);
-        } elseif ($this->_encrypted) {
-            Message::encrypted($this->_to, $this->_content, $this->_html, $this->_id, $this->_replace);
+            Muc::message($this->_to, $this->_content, $this->_html, $this->_id, $this->_file, $this->_attachid, [], $this->_originid);
         } else {
-            Message::message($this->_to, $this->_content, $this->_html, $this->_id, $this->_replace, $this->_file);
+            Message::message($this->_to, $this->_content, $this->_html, $this->_id, $this->_replace, $this->_file, $this->_attachid, [], $this->_originid);
         }
     }
 
@@ -47,4 +46,10 @@ class Publish extends Action
             $m->handle($stanza, $parent);
         }
     }
+
+    public function error($id, $message = '')
+    {
+        $this->pack($message);
+        $this->deliver();
+    }
 }
