@@ -2,7 +2,6 @@
 
 namespace Moxl\Xec\Action\Pubsub;
 
-use Moxl\Xec\Action;
 use Moxl\Stanza\Pubsub;
 use Moxl\Xec\Action\Pubsub\Errors;
 
@@ -18,6 +17,12 @@ class SetConfig extends Errors
         Pubsub::setConfig($this->_to, $this->_node, $this->_data);
     }
 
+    public function error($id, $message = '')
+    {
+        $this->pack($message);
+        $this->deliver();
+    }
+
     public function handle($stanza, $parent = false)
     {
         $this->deliver();
