@@ -2,7 +2,6 @@
 
 namespace Moxl\Xec\Action\PubsubSubscription;
 
-use Moxl\Xec\Action;
 use Moxl\Xec\Action\Pubsub\Errors;
 use Moxl\Stanza\PubsubSubscription;
 
@@ -18,8 +17,11 @@ class Add extends Errors
     {
         $this->store();
         PubsubSubscription::listAdd(
-            $this->_server, $this->_from, $this->_node,
-            $this->_data['title'], $this->_pepnode
+            $this->_server,
+            $this->_from,
+            $this->_node,
+            $this->_data['title'],
+            $this->_pepnode
         );
     }
 
