@@ -2,7 +2,6 @@
 
 namespace Moxl\Xec\Action\PubsubSubscription;
 
-use Moxl\Xec\Action;
 use Moxl\Xec\Action\Pubsub\Errors;
 use Moxl\Stanza\PubsubSubscription;
 
@@ -17,7 +16,10 @@ class Remove extends Errors
     {
         $this->store();
         PubsubSubscription::listRemove(
-            $this->_server, $this->_from, $this->_node, $this->_pepnode
+            $this->_server,
+            $this->_from,
+            $this->_node,
+            $this->_pepnode
         );
     }
 
