@@ -16,12 +16,14 @@ class Carbons extends Payload
                 $m = \App\Message::findByStanza($message);
                 $m->set($message, $stanza->forwarded);
 
-                if (!$m->isOTR()) {
+                if (!$m->encrypted) {
                     $m->save();
+                    $m->clearUnreads();
+
                     $this->pack($m);
                     $this->deliver();
                 }
-            } else if ($message->displayed) {
+            } elseif ($message->displayed) {
                 // Another client just displayed the message
                 $displayed = new Displayed;
                 $displayed->handle($message->displayed, $message);
