@@ -27,9 +27,9 @@ class Migration extends AbstractMigration
             case 'mysql':
                 $this->schema->getConnection()->unprepared('SET foreign_key_checks = 1');
                 break;
-            case 'sqlite':
+            /*case 'sqlite':
                 $this->schema->getConnection()->unprepared('PRAGMA foreign_keys = on');
-                break;
+                break;*/
         }
     }
 
@@ -39,9 +39,9 @@ class Migration extends AbstractMigration
             case 'mysql':
                 $this->schema->getConnection()->unprepared('SET foreign_key_checks = 0');
                 break;
-            case 'sqlite':
+            /*case 'sqlite':
                 $this->schema->getConnection()->unprepared('PRAGMA foreign_keys = off');
-                break;
+                break;*/
         }
     }
 }
